#ifndef BPPROBJ_H
#define BPPROBJ_H

#include <bppdef.h>

/** Base of reference counted classes.
This abstract class is used for base of all classes to be reference counted, 
i.e. all non trivial data objects. It implements reference counting mechanism.
*/
class BPPLIB_DLLEXP BppRefObject
{
public:
   /// dtor - does nothing
   virtual ~BppRefObject();

   /** Adds reference.
   Function increments internal reference counter.
   @return number of references (after increment)
   */
   inline long addRef()
   {
      return ++_refCnt;
   } 
   
   /// release object reference
   long release();

   /** Gets current number of references.
   @return current number of references to object
   */
   inline long getRefCnt() const
   {
      return _refCnt;
   }

   /** Sets number of references explicitely.
   This function is internally used by bytecode interpreter. Client code normally should never need
   to call it.
   @param refCnt number of references
   @see BppInterpreter::doOP_MKREF()
   */
   inline void setRefCnt(long refCnt)
   {
      _refCnt = refCnt;
   }
protected:
   /// default ctor - protected to make an abstract class, initializes _refCnt to zero.
   BppRefObject();
   /// copy ctor - protected, initializes _refCnt to zero. 
   BppRefObject(const BppRefObject&);
private:
   /// reference counter
   long _refCnt; 
};
#endif // BPPROBJ_H
