
/** @file
Defines implementation functions for B++ predefined functions.
*/ 

#ifndef DOXY
#ifdef WIN32
#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0601
#endif
#endif

#include <stdlib.h>
#include <stdio.h>
#ifndef WINCE
 #include <time.h>
 #ifndef GCC
 #include <process.h>
 #else
   extern char** environ;
 #endif
#endif


#include "bppufunc.h"
#include "bppdfunc.h"
#include "bppbuf.h"
#include "bppvm.h"
#include "bppint.h"
#include "bppvalue.h"
#include "bppobjct.h"
#include "bppstrng.h"
#include "bppvec.h"
#include "bppprint.h"
#include "bppfimpl.h"

#ifdef WIN32
   #include <windows.h>
#endif
#ifdef __linux__
#include <dlfcn.h>
#endif

#ifdef __MSDOS__
#include <dos.h>
#ifndef __TURBOCC__
   #include <conio.h>
#endif
#include "usedll.h"
   static union REGS _regs;
   static struct SREGS _sregs;



#define SWITCHREGSET(NAME,PREFIX) if (_tcsicmp(#NAME,regname) == 0) {PREFIX##.##NAME = val; return true;}
static bool doSetReg(const BppChar* regname, BppWord val)
{
        SWITCHREGSET(ax,_regs.x)
        SWITCHREGSET(bx,_regs.x)
        SWITCHREGSET(cx,_regs.x)
        SWITCHREGSET(dx,_regs.x)
        SWITCHREGSET(si,_regs.x)
        SWITCHREGSET(di,_regs.x)
#ifndef __WATCOMC__
        SWITCHREGSET(cflag,_regs.x)
#endif
#ifdef __TURBOC__
        SWITCHREGSET(flags,_regs.x)
#endif
        SWITCHREGSET(al,_regs.h)
        SWITCHREGSET(ah,_regs.h)
        SWITCHREGSET(bl,_regs.h)
        SWITCHREGSET(bh,_regs.h)
        SWITCHREGSET(cl,_regs.h)
        SWITCHREGSET(ch,_regs.h)
        SWITCHREGSET(dl,_regs.h)
        SWITCHREGSET(dh,_regs.h)
        SWITCHREGSET(ds,_sregs)
        SWITCHREGSET(es,_sregs)
        SWITCHREGSET(ss,_sregs)
        SWITCHREGSET(cs,_sregs)
   return false;
}
// setreg(string regname1[, string regname2],int val)
bool BppDefaultFunctions::Bpp_setreg(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(2,3);
   ValPtr bp = sp; bp -= argc;
   if (!(bp[argc].isIntegralType()))
     THROW(_T("invalid type of value argument"));
   BppString* s = bp[argc-1];
   if (s == 0)
     THROW(_T("invalid type of register argument"));
   BppInt val = *sp;
   if (!doSetReg(s->c_str(),(BppWord) (val & 0xFFFF)))
               THROW(_T("invalid register name"));
   if (argc == 3)
   {
      s = bp[1];
      if (s == 0)
         THROW(_T("invalid type of register argument"));
      if (!doSetReg(s->c_str(),(BppWord) (val >> 16)))
         THROW(_T("invalid register name"));
   }
   sp = bp;
   sp->dispose();
   return true;
}

#define SWITCHREGGET(NAME,PREFIX) if (_tcsicmp(#NAME,regname) == 0) { val = (BppWord) PREFIX##.##NAME ; return true;}
static bool doGetReg(const BppChar* regname, BppWord& val)
{
        SWITCHREGGET(ax,_regs.x)
        SWITCHREGGET(bx,_regs.x)
        SWITCHREGGET(cx,_regs.x)
        SWITCHREGGET(dx,_regs.x)
        SWITCHREGGET(si,_regs.x)
        SWITCHREGGET(di,_regs.x)
#ifndef __WATCOMC__
        SWITCHREGGET(cflag,_regs.x)
#endif
#ifdef __TURBOC__
        SWITCHREGGET(flags,_regs.x)
#endif
        SWITCHREGGET(al,_regs.h)
        SWITCHREGGET(ah,_regs.h)
        SWITCHREGGET(bl,_regs.h)
        SWITCHREGGET(bh,_regs.h)
        SWITCHREGGET(cl,_regs.h)
        SWITCHREGGET(ch,_regs.h)
        SWITCHREGGET(dl,_regs.h)
        SWITCHREGGET(dh,_regs.h)
        SWITCHREGGET(ds,_sregs)
        SWITCHREGGET(es,_sregs)
        SWITCHREGGET(ss,_sregs)
        SWITCHREGGET(cs,_sregs)
   return false;
}
// intval = getreg(string regname1[, string regname2])
bool BppDefaultFunctions::Bpp_getreg(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(1,2);
   ValPtr bp = sp; bp -= argc;
   BppString* s = bp[1];
   if (s == 0)
     THROW(_T("invalid type of register argument"));
   BppWord regval;
   BppDWord val=0;
   if (!doGetReg(s->c_str(),regval))
       THROW(_T("invalid register name"));
   val = regval;
   if (argc == 2)
   {
      val = val << 16;
      s = bp[2];
      if (s == 0)
         THROW(_T("invalid type of register argument"));
      if (!doGetReg(s->c_str(),regval))
         THROW(_T("invalid register name"));
      val += regval;
   }
   *bp = (BppInt) val;
   sp = bp;
   return true;
}

// segread()
bool BppDefaultFunctions::Bpp_segread(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(0,0);
        segread(&_sregs);
   sp->dispose();
   return true;
}

// int axval = int86(int intnr)
bool BppDefaultFunctions::Bpp_int86(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(1,1);
   if (!sp->isIntegralType())
     THROW(_T("invalid argument type"));
        BppUShort n =  *sp;
        --sp;
   *sp = (BppInt) int86(n,&_regs,&_regs);
   return true;
}

// int axval = int86x(int intnr)
bool BppDefaultFunctions::Bpp_int86x(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(1,1);
   if (!sp->isIntegralType())
     THROW(_T("invalid argument type"));
        BppUShort n = *sp;
        --sp;
   *sp = (BppInt) int86x(n,&_regs,&_regs,&_sregs);
   return true;
}

// inbyte = inportb(int port)
bool BppDefaultFunctions::Bpp_inportb(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(1,1);
   if (!sp->isIntegralType())
     THROW(_T("invalid argument type"));
        BppUShort port = *sp;
        --sp;
#ifdef inportb
  #undef inportb
#endif
#ifndef __TURBOC__
   #ifdef _MSC_VER
      #define inportb _inp
   #else
       #define inportb inp
   #endif
#endif
        *sp = (BppByte) inportb((unsigned)port);
   return true;
}

// void outportb(port,byteval)
bool BppDefaultFunctions::Bpp_outportb(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(2,2);
   ValPtr bp = sp; bp -= 2;    if (!sp->isIntegralType())

   if (!(bp[1].isIntegralType() && bp[2].isIntegralType()))
          THROW(_T("invalid argument type"));
#ifdef outportb
  #undef outportb
#endif
#ifndef __TURBOC__
   #ifdef _MSC_VER
      #define outportb _outp
   #else
       #define outportb outp
   #endif
#endif
                  outportb((BppUShort) bp[1],(BppByte) bp[2]);
   sp = bp;
   sp->dispose();
   return true;
}

// inword = inport(int port)
bool BppDefaultFunctions::Bpp_inport(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
#ifdef inport
  #undef inport
#endif
#ifndef __TURBOC__
   #ifdef _MSC_VER
      #define inport _inpw
   #else
       #define inport inpw
   #endif
#endif
   CHKVM;
   CHKARGC(1,1);
   if (!sp->isIntegralType())
     THROW(_T("invalid argument type"));
        BppUShort port = *sp;
   --sp;
   BppUShort v = inport(port);
   *sp = v;
   return true;
}

// void outportb(port,wordval)
bool BppDefaultFunctions::Bpp_outport(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
#ifdef outport
  #undef outport
#endif
#ifndef __TURBOC__
   #ifdef _MSC_VER
      #define outport _outpw
   #else
       #define outport outpw
   #endif
#endif
   CHKVM;
   CHKARGC(2,2);
   ValPtr bp = sp; bp -= 2;
   if (!(bp[1].isIntegralType() && bp[2].isIntegralType()))
     THROW(_T("invalid argument type"));
        outport((BppUShort) bp[1],(BppUShort) bp[2]);
   sp = bp;
   sp->dispose();
   return true;
}



#endif // __MSDOS__

#ifdef WIN32  // Borrowed from MSDN
// https://docs.microsoft.com/en-us/windows/console/clearing-the-screen

void cls(HANDLE hConsole)
{
   COORD coordScreen = { 0, 0 };    // home for the cursor 
   DWORD cCharsWritten;
   CONSOLE_SCREEN_BUFFER_INFO csbi;
   DWORD dwConSize;

   // Get the number of character cells in the current buffer. 

   if (!GetConsoleScreenBufferInfo(hConsole, &csbi))
   {
      return;
   }

   dwConSize = csbi.dwSize.X * csbi.dwSize.Y;

   // Fill the entire screen with blanks.

   if (!FillConsoleOutputCharacter(hConsole,        // Handle to console screen buffer 
      (TCHAR) ' ',     // Character to write to the buffer
      dwConSize,       // Number of cells to write 
      coordScreen,     // Coordinates of first cell 
      &cCharsWritten))// Receive number of characters written
   {
      return;
   }

   // Get the current text attribute.

   if (!GetConsoleScreenBufferInfo(hConsole, &csbi))
   {
      return;
   }

   // Set the buffer's attributes accordingly.

   if (!FillConsoleOutputAttribute(hConsole,         // Handle to console screen buffer 
      csbi.wAttributes, // Character attributes to use
      dwConSize,        // Number of cells to set attribute 
      coordScreen,      // Coordinates of first cell 
      &cCharsWritten)) // Receive number of characters written
   {
      return;
   }

   // Put the cursor at its home coordinates.

   SetConsoleCursorPosition(hConsole, coordScreen);
}

#endif
// graphic bios functions
// int newmode =  setscrmode(int mode)
bool BppDefaultFunctions::Bpp_setscrmode(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(1, 1);
   if (!sp->isIntegralType())
      THROW(_T("invalid argument type"));
#ifdef __MSDOS__
   union REGS regs;
   regs.h.ah = 0;
   regs.h.al = (BppByte)*sp;
   int86(0x10, &regs, &regs);
   regs.h.ah = 0x0F;
   int86(0x10, &regs, &regs);
   --sp;
   *sp = (BppByte)regs.h.al;;
#else
   BppInt mode = *sp;
   sp[-1] = mode;
    --sp;
#ifdef WIN32

    HANDLE hStdout = GetStdHandle(STD_OUTPUT_HANDLE);

   if (hStdout) {
      cls(hStdout);
      CONSOLE_CURSOR_INFO info;
      if (GetConsoleCursorInfo(hStdout, &info))
      {
         info.bVisible = (mode == 7) ? 1 : 0;
         SetConsoleCursorInfo(hStdout, &info);
      }
   }
#endif
#endif
   return true;
}

// void setpixel(x, y, int color)
bool BppDefaultFunctions::Bpp_setpixel(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(3, 3);
   ValPtr bp = sp; bp -= 3;
   if (!sp->isIntegralType())
      THROW(_T("invalid argument type"));
#ifdef __MSDOS__
   union REGS regs;
   regs.x.cx = (BppUShort)bp[1];
   regs.x.dx = (BppUShort)bp[2];
   regs.h.al = (BppByte)*sp;
   regs.h.ah = 0x0c;
   int86(0x10, &regs, &regs);
#else
#ifdef WIN32
   BppUInt color = *sp;
   HWND console_handle = GetConsoleWindow();
   if (console_handle) {
      HDC device_context = GetDC(console_handle);

      HPEN pen = CreatePen(PS_SOLID, 1, (COLORREF)color);
      SelectObject(device_context, pen);
      SetPixel(device_context, (BppUShort)bp[1], (BppUShort)bp[2], (COLORREF)color);

      ReleaseDC(console_handle, device_context);
   }
#endif

#endif
   sp = bp;
   sp->dispose();
   return true;
}

// void line(x1,y1,x2,y2,int color)
bool BppDefaultFunctions::Bpp_line(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(5, 5);
   if (!sp->isIntegralType())
      THROW(_T("invalid argument type"));
   ValPtr bp = sp; bp -= 5;
   BppShort x1 = bp[1];
   BppShort y1 = bp[2];
   BppShort x2 = bp[3];
   BppShort y2 = bp[4];
#ifdef  __MSDOS__
   union REGS regs, r2;
   int i;
   regs.h.ah = 0x0c;
   regs.h.al = (BppByte)*sp;

   register int dx = x2 - x1;
   register int dy = y2 - y1;
   register int dist, incx, incy, xerr, yerr;

   /* increment and sign for both directions */
   if (dx < 0)
   {
      incx = -1;
      dx = -dx;
   }
   else
      incx = (dx != 0) ? 1 : 0;

   if (dy < 0)
   {
      incy = -1;
      dy = -dy;
   }
   else
      incy = (dy != 0) ? 1 : 0;

   /* largest distance */
   dist = (dx>dy) ? dx : dy;

   /* init */
   regs.x.cx = x1;
   regs.x.dx = y1;
   xerr = dx;
   yerr = dy;

   /* draw pixels */
   for (i = 0; i<dist; i++)
   {
      int86(0x10, &regs, &r2);

      xerr += dx;
      yerr += dy;

      if (xerr>dist)
      {
         xerr -= dist;
         regs.x.cx += incx;
      }
      if (yerr>dist)
      {
         yerr -= dist;
         regs.x.dx += incy;
      }
   }

   regs.x.cx = x2;
   regs.x.dx = y2;
   int86(0x10, &regs, &r2);
#else
#ifdef WIN32
   BppUInt color = *sp;
   HWND console_handle = GetConsoleWindow();
   if (console_handle) {
      HDC device_context = GetDC(console_handle);

      //Here's a 5 pixels wide RED line [from initial 0,0] to 300,300
      HPEN pen = CreatePen(PS_SOLID, 1, (COLORREF)color);
      SelectObject(device_context, pen);
      MoveToEx(device_context, x1, y1, 0);
      LineTo(device_context, x2,y2);


      ReleaseDC(console_handle, device_context);
   }
#endif
#endif
   sp = bp;
   sp->dispose();
   return true;
}


   // miscellanous funcs
// rnd = rand()
bool BppDefaultFunctions::Bpp_rand(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   ARGCMAX(0);
   *sp = (BppInt) rand();
   return true;
}

// srand(int seed)
bool BppDefaultFunctions::Bpp_srand(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(1,1);
   srand((unsigned) (BppInt) *sp);
   --sp;
   sp->dispose();
   return true;
}

bool BppDefaultFunctions::Bpp_ctime(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(1,1);
   if (!sp->isIntegralType())
      THROW(_T("invalid argument type"));
   time_t tt = /*(time_t) (BppInt)*/ *sp;
   BppChar* p = _tctime(&tt);

   --sp;
   *sp = BppString(p);
   return true;
}

// tmvec = localtime(time_t)
bool BppDefaultFunctions::Bpp_localtime(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
//#if (!WINCE) && (_MSC_VER >= 1400)
//#define ltimefunc _localtime32
//#define _timet __time32_t
//#else
#define ltimefunc localtime
#define _timet time_t
//#endif
   CHKVM;
   CHKARGC(1,1);
   if (!sp->isIntegralType())
      THROW(_T("invalid argument type"));
   _timet tt = /*(_timet) (BppInt)*/ *sp;
   tm* ptm = ltimefunc(&tt);
   BppVector* tmvec = new BppVector(9,true);
   int* p = (int*) ptm;
   for(int i=0;i<9;i++,p++)
      (*tmvec)[i] = (BppInt) (int) *p;
   --sp;
   *sp = tmvec;
   return true;
}
// argvec = getargs()
bool BppDefaultFunctions::Bpp_getargs(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(0,0);
   BppVector* pvec = new BppVector();
   for (int i=0; i<vm->cmdargc(); i++)
      pvec->push_back(new BppString((const BppChar*) vm->cmdargv()[i]));
   *sp = pvec;
   return true;
}

// argvec = getusrargs()
bool BppDefaultFunctions::Bpp_getusrargs(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   ARGCMAX(0);
   BppVector* pvec = new BppVector();
   bool found = false;
   for (int i=0; i<vm->cmdargc(); i++)
   {
      BppChar* p = vm->cmdargv()[i];
      if ((!found) && (p[0] == _T('#')))
      {
         found = true;
         p++;
         if (*p == 0) continue;
      }
      if (found)
         pvec->push_back(new BppString((const BppChar*) p));
   }
   *sp = pvec;
   return true;
}

bool BppDefaultFunctions::Bpp_errno(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   ARGCMAX(1);
#ifdef WINCE
   if (argc) { SetLastError((BppUInt)*sp); --sp; }
   *sp = (BppInt) GetLastError();

#else
   if (argc) { errno = (BppInt)*sp; --sp; }
   *sp = (BppInt) errno;
#endif
   return true;
}

bool BppDefaultFunctions::Bpp_strerror(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
#ifdef WINCE
	return Bpp_GetSysErrorMsg(vm,argc,sp);
#else
   CHKVM;
   ARGCMAX(1);
   int nr = errno;
   if (argc) { nr = (BppInt) *sp; --sp; }
   if (nr >= 0 && nr < sys_nerr) *sp = new BppString(_tcserror(nr));
   else *sp = new BppString(_T("Unknown error"));
   return true;
#endif
}

bool BppDefaultFunctions::Bpp_nerr(BppVirtualMachine * vm, int argc, ValPtr & sp)
{
   CHKVM;
   ARGCMAX(0);
   *sp = (BppInt)
#ifdef WINCE
	   0;
#else
	   sys_nerr;
#endif
   return true;
}

// cnt = argcnt()
bool BppDefaultFunctions::Bpp_argcnt(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   ARGCMAX(0);
   BppProcInfo* pinfo = vm->interpreter()->getProcInfo();
   BppInt cnt = -1;
   if (pinfo != 0)
   {
      cnt = pinfo->getNargs();
   }
   *sp = cnt;
   return true;
}

bool BppDefaultFunctions::Bpp_arg(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(1,1);
   if (!sp->isIntegralType())
      THROW(_T("invalid argument type"));
   BppProcInfo* pinfo = interpreter->getProcInfo();
   BppInt cnt = -1;
   BppInt idx = *sp;
   if (pinfo != 0)
   {
      cnt = pinfo->getNargs();
   }
   if ((idx < 0) || (idx >= cnt))
      THROW(_T("invalid index"));
   --sp;
   *sp = pinfo->bp()[(int) (idx+1)];
   return true;
}

bool BppDefaultFunctions::Bpp_argv(BppVirtualMachine * vm, int argc, ValPtr & sp)
{
   CHKVM;
   ARGCMAX(0);
   BppProcInfo* pinfo = interpreter->getProcInfo();
   BppInt cnt = -1;
   if (pinfo != 0)
   {
      cnt = pinfo->getNargs();
   }
   BppVector* pvec = (cnt > 0) ? new BppVector((unsigned)cnt,true) : new BppVector();
   *sp = pvec;
   for (int i = 0; i < cnt; ++i)
      (*pvec)[i] = pinfo->bp()[i + 1];
   return true;
}

bool BppDefaultFunctions::Bpp_call(BppVirtualMachine * vm, int argc, ValPtr & sp)
{
   CHKVM;
   CHKARGC(1, 2);
   ValPtr bp = sp; bp -= 2;
   const BppValue& func = bp[1];
   BppVector* args = 0;
   if (argc == 2)
   {
      args = bp[2];
      if (!args) THROW(_T("invalid second argument - vector expected"));
   }
   BppObject* pobj = func;
   bool ok = false;
   if (pobj) ok = interpreter->invokeMember(*bp, pobj, _T("OP_CALL"), args);
   else ok = interpreter->invokeFunc(*bp, func, args);
   sp = bp;
   return ok;
}


   // system functions
// ptr = addr(val)
bool BppDefaultFunctions::Bpp_addr(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(1,1);
   BppValueType vt = sp->getType();
   if (vt == BVT_NULL)
      THROW(_T("argument is NULL"));
   if (vt >= BVT_INTEGRAL_MIN && vt <= BVT_REAL_MAX)
      return Bpp_intptr(vm, argc, sp);
   else 
      sp[-1] = 
#ifdef GCC
    (BppLong)
#else
    (BppIntPtr) 
#endif

   sp->addr();
   --sp;
   return true;
}

bool BppDefaultFunctions::Bpp_memmove(BppVirtualMachine * vm, int argc, ValPtr & sp)
{
   static const BppChar* fname = _T("memmove");
   CHKVM;
   CHKARGC(3, 3);
   ValPtr bp = sp; bp -= 3;
   BppValue v;
   if (!interpreter->doNumCast(v, bp[1], BVT_INTPTR, fname)) return false;
   BppIntPtr dst = (BppIntPtr) v.ptr();
   if (!interpreter->doNumCast(v, bp[2], BVT_INTPTR, fname)) return false;
   BppIntPtr src = (BppIntPtr)v.ptr();
   if (!interpreter->doNumCast(v, bp[3], BVT_UINT, fname)) return false;
   BppUInt n = v;
#ifndef NOCPPEXCEPTION

   try
   {
#endif
   dst = movemem(dst, src, n);
#ifndef NOCPPEXCEPTION
   }
   catch (...)
   {
      return interpreter->returnError(_T("fatal error"),fname);
   }
#endif

   sp = bp;
   *sp = dst;
   return true;
}

bool BppDefaultFunctions::Bpp_getenv( BppVirtualMachine* vm, int argc, ValPtr& sp )
{
   CHKVM;
   ARGCMAX(1);
   if (argc > 0)
   {
	   BppString* key = *sp;
	   if (key == 0) THROW(_T("invalid argument type - string expected"));
	   const BppChar* presult = _tgetenv(key->c_str());
	   --sp;
	   (*sp) = new BppString(presult);
   }
   else
   { 
	   BppDictionary* dict = new BppDictionary(true);
       (*sp) = dict;
#ifndef WINCE
	   BppChar** penv = environ;
	   for(;(*penv);++penv)
	   {
		   BppString s = *penv;
		   long pos = s.find(_T('='));
		   if (pos>0)
		   {
			   (*dict)[s.substr(0,pos)] = new BppString(s.substr(pos+1));
		   }
	   }
#endif
   }
   return true;
}

bool BppDefaultFunctions::Bpp_setenv( BppVirtualMachine* vm, int argc, ValPtr& sp )
{
   CHKVM;
   CHKARGC(2,2);
   ValPtr bp = sp; bp -= 2;
   BppString* key = bp[1];
   BppString* val = bp[2];
   if ((key == 0) || (val == 0)) THROW(_T("invalid argument type - string expected"));
   size_t l1 = key->length();
   int succ = 0;
   if (l1 > 0)
   {
      size_t l2 = val->length();
      BppChar* buf = (BppChar*) allocmem((l1+l2+2) * sizeof(BppChar));
      _tcscpy(buf,key->c_str());
      _tcscat(buf,_T("="));
      _tcscat(buf,val->c_str());
      if (_tputenv(buf) == 0) succ = 1;
      freemem(buf);
   }
   sp = bp;
   (*sp) = (BppInt) succ;
   return true;
}

bool BppDefaultFunctions::Bpp_setlocale(BppVirtualMachine * vm, int argc, ValPtr & sp)
{
	CHKVM;
	CHKARGC(2, 2);
	ValPtr bp = sp; bp -= 2;
	BppValue v;
	if (!interpreter->doNumCast(v,bp[1],BVT_INT,_T("setlocale"))) return false;
	BppInt category = v;
	const BppChar* strloc = 0;
	if (sp->getType() != BVT_NULL) {
	   v.dispose();	
	   if (!interpreter->doStrCast(v,bp[2])) THROW(_T("invalid argument type - string expected"));
	   BppString* s = v;
	   if (s) strloc = s->c_str();
	}
	strloc = _tsetlocale(category, strloc);
	sp = bp;
	*sp = new BppString(strloc);
	return true;
}

#ifndef WINCE
bool BppDefaultFunctions::Bpp_system(BppVirtualMachine * vm, int argc, ValPtr & sp)
{
	CHKVM;
	CHKARGC(1, 1);
	BppString* s = *sp;
	if (s == 0) THROW(_T("invalid argument type - string expected"));
	int r = _tsystem(s->c_str());
	if (r == -1) THROW(_T("System call failed."), s->c_str());
	*(--sp) = (BppInt)r;
	return true;
}
#endif

#ifdef __MSDOS__
   typedef HDLL HMODULE;
   typedef int BOOL;
   typedef long (huge *FARPROC)();
#endif // __MSDOS__
// int hinst = LoadLibrary(dllname)
bool BppDefaultFunctions::Bpp_LoadLibrary(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(1,1);
   BppString* s = *sp;
   if (s == 0)
     THROW(_T("invalid argument type"));
   --sp;
   HMODULE h = 0;
#ifndef NOCPPEXCEPTION

   try
   {
#endif
	   BppString _dllname = *s;
	   _dllname.replace(_T("*"),BPP_DLLWILD);
	   h = LoadLibrary(s->c_str());
#ifndef NOCPPEXCEPTION
   }
   catch (...)
   {
      h = 0;
   }
#endif
   *sp = (BppIntPtr) h;
   return true;
}

// ok = FreeLibrary(intptr hmodule)
bool BppDefaultFunctions::Bpp_FreeLibrary(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(1,1);
   if (!sp->isIntegralType())
     THROW(_T("invalid argument type"));
   BOOL result = 0;
#ifndef NOCPPEXCEPTION
   try
   {
#endif
      result = FreeLibrary((HMODULE) (BppIntPtr) *sp);
#ifdef __MSDOS__
      result = (result< 0) ? 0 : 1;

#endif
#ifndef NOCPPEXCEPTION
   }
   catch (...)
   {
      result = 0;
   }
#endif
   --sp;
   *sp = (BppInt) result;
   return true;
}

#ifndef __MSDOS__
#ifdef _UNICODE
#define LIBREPLACE 'W'
#else
#define LIBREPLACE 'A'
#endif
#endif
// int procaddr = GetProcAddress(intptr hmodule, string name)
bool BppDefaultFunctions::Bpp_GetProcAddress(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   CHKARGC(2,2);
   ValPtr bp = sp;
   bp -= 2;
   if (!bp[1].isIntegralType())
      THROW(_T("invalid argument type"));
   BppString* s = bp[2];
   if (s == 0)
      THROW(_T("invalid argument type"));
#ifndef __MSDOS__
   BppString namestr(*s);
   namestr.replace(_T('?'),_T(LIBREPLACE));
   s = &namestr;
#endif
#ifndef NOCPPEXCEPTION
   try
   {
#endif
      *bp = (BppIntPtr) GetProcAddress((HMODULE) (BppIntPtr) bp[1], s->p_str());
#ifndef NOCPPEXCEPTION
   }
   catch (...)
   {
      *bp = (BppIntPtr) 0;
   }
#endif
   sp = bp;
   return true;
}

#ifdef WIN32
   #define _ole32dll _T("ole32.dll")
 #ifndef WINCE
   #define _user32dll _T("user32.dll")
   #define _shell32dll _T("shell32.dll")
   #define _kernel32dll _T("kernel32.dll")
   #define _gdi32dll _T("gdi32.dll")
 #else
   #define _user32dll _T("coredll.dll")
   #define _shell32dll _T("coredll.dll")
   #define _kernel32dll _T("coredll.dll")
   #define _gdi32dll _T("coredll.dll")
   #define _user32 _coredll
   #define _kernel32 _coredll
   #define _gdi32 _coredll
   #define _shell32 _coredll
 #endif

static struct LibHandles
{
#ifndef WINCE
   HMODULE _user32;
   HMODULE _kernel32;
   HMODULE _gdi32;
   HMODULE _shell32;
#else
   HMODULE _coredll;
#endif
   HMODULE _ole32;
   LibHandles():_ole32(0),
#ifndef WINCE
   _user32(0),_kernel32(0),_gdi32(0),_shell32(0)
#else
   _coredll(0)
#endif
   {
   }

   ~LibHandles()
   {
      if (_ole32 != 0) FreeLibrary(_ole32);
#ifndef WINCE
      if (_user32 != 0) FreeLibrary(_user32);
      if (_shell32 != 0) FreeLibrary(_shell32);
      if (_gdi32 != 0) FreeLibrary(_gdi32);
      if (_kernel32 != 0) FreeLibrary(_kernel32);
#else
      if (_coredll != 0) FreeLibrary(_coredll);
#endif
   }

}
_libHandles;

#ifndef DOXY
static bool GetDefaultLibFunc(BppVirtualMachine* vm, int argc, ValPtr& sp, HMODULE& libHandle, const BppChar* libName)
{
   CHKVM;
   CHKARGC(1, 1);
   ValPtr bp = sp;
   bp -= argc;
   BppProcInfo* pi = interpreter->getProcInfo();
   BppClass* ownClass = pi ? pi->getClass() : 0;
   BppString* s = interpreter->getStringFromValue(ownClass,bp[1]);
   if (s == 0)
      THROW(_T("invalid argument type"));
   BppString namestr(*s);
   namestr.replace(_T('?'), _T(LIBREPLACE));
   s = &namestr;
   if (libHandle == 0)
   {
      libHandle = LoadLibrary(libName);
   }
#ifndef NOCPPEXCEPTION
   try
   {
#endif
      *bp = (BppIntPtr)GetProcAddress(libHandle, s->p_str());
#ifndef NOCPPEXCEPTION
   }
   catch (...)
   {
      *bp = (BppIntPtr)0;
   }
#endif
   sp = bp;
   return true;

}
#endif

bool BppDefaultFunctions::Bpp_GetUser32Func(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   return GetDefaultLibFunc(vm, argc, sp, _libHandles._user32, _user32dll);
}

bool BppDefaultFunctions::Bpp_GetKernel32Func(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   return GetDefaultLibFunc(vm, argc, sp, _libHandles._kernel32, _kernel32dll);
}

bool BppDefaultFunctions::Bpp_GetGdi32Func(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   return GetDefaultLibFunc(vm, argc, sp, _libHandles._gdi32, _gdi32dll);
}

bool BppDefaultFunctions::Bpp_GetShell32Func(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   return GetDefaultLibFunc(vm, argc, sp, _libHandles._shell32, _shell32dll);
}

bool BppDefaultFunctions::Bpp_GetOle32Func(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   return GetDefaultLibFunc(vm, argc, sp, _libHandles._ole32, _ole32dll);
}

bool BppDefaultFunctions::Bpp_GetLastError(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   ARGCMAX(0);
   *sp = (BppUInt) GetLastError();
   return true;
}

bool BppDefaultFunctions::Bpp_GetSysErrorMsg(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   ARGCMAX(1);
   ValPtr bp = sp;
   bp -= argc;
   DWORD errcode = (argc > 0) ? ((DWORD) (BppUInt) bp[1]) : GetLastError();

   LPVOID lpMsgBuf = 0;
   FormatMessage(
      FORMAT_MESSAGE_ALLOCATE_BUFFER |
      FORMAT_MESSAGE_FROM_SYSTEM |
      FORMAT_MESSAGE_IGNORE_INSERTS,
      NULL,
      errcode,
      MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
      (LPTSTR) &lpMsgBuf,
      0,
      NULL
      );
   if (lpMsgBuf != 0)
   {
     *bp = new BppString((const BppChar*) lpMsgBuf);
     LocalFree(lpMsgBuf);
   }
   else
      *bp = new BppString();
   sp = bp;
   return true;
}

#endif // WIN32

#ifdef NOASM
static BppIntPtr CallLibFuncCE(FARPROC proc,int argc, BppIntPtr* argv)
{
   switch (argc)
   {
   case 0:
      return ((DLLFUNC0)proc)();
   case 1:
      return ((DLLFUNC1)proc)(argv[0]);
   case 2:
      return ((DLLFUNC2)proc)(argv[0],argv[1]);
   case 3:
      return ((DLLFUNC3)proc)(argv[0],argv[1],argv[2]);
   case 4:
      return ((DLLFUNC4)proc)(argv[0],argv[1],argv[2],argv[3]);
   case 5:
      return ((DLLFUNC5)proc)(argv[0],argv[1],argv[2],argv[3],argv[4]);
   case 6:
      return ((DLLFUNC6)proc)(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
   case 7:
      return ((DLLFUNC7)proc)(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
   case 8:
      return ((DLLFUNC8)proc)(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7]);
   case 9:
      return ((DLLFUNC9)proc)(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],
         argv[8]);
   case 10:
      return ((DLLFUNC10)proc)(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],
         argv[8],argv[9]);
   case 11:
      return ((DLLFUNC11)proc)(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],
         argv[8],argv[9],argv[10]);
   case 12:
      return ((DLLFUNC12)proc)(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],
         argv[8],argv[9],argv[10],argv[11]);
   case 13:
      return ((DLLFUNC13)proc)(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],
         argv[8],argv[9],argv[10],argv[11],argv[12]);
   case 14:
      return ((DLLFUNC14)proc)(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],
         argv[8],argv[9],argv[10],argv[11],argv[12],argv[13]);
   case 15:
      return ((DLLFUNC15)proc)(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],
         argv[8],argv[9],argv[10],argv[11],argv[12],argv[13],argv[14]);
   case 16:
      return ((DLLFUNC16)proc)(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],
         argv[8],argv[9],argv[10],argv[11],argv[12],argv[13],argv[14],argv[15]);
   }
   return 0;
}
#endif

#ifdef __WATCOMC__
#define asm _asm
#endif
// result = CallLibFunc(procadr,...)
bool BppDefaultFunctions::Bpp_CallLibFunc(BppVirtualMachine* vm, int argc, ValPtr& sp)
{
   CHKVM;
   ARGCMIN(1);
   // TODO: Anpassung für x64
#ifdef NOASM
   ARGCMAX(17);
#endif
   ValPtr vp = sp;
   vp -= argc;
   if (!vp[1].isIntegralType())
      THROW(_T("invalid argument type"));
   BppIntPtr* pargs =(BppIntPtr*) allocmem(sizeof(BppIntPtr) *argc);
#ifndef NOASM
   for (int i=0; i<argc;i++)
      pargs[argc-(i+1)] = (BppIntPtr) vp[i+1].ptr();
   FARPROC proc = (FARPROC) pargs[argc-1];
#else
   for (int i=0; i<argc;i++)
      pargs[i] = (BppIntPtr)vp[i + 1].ptr();
   FARPROC proc = (FARPROC) pargs[0];
#endif
   int procargc = argc-1;
#ifndef __MSDOS__
   BppIntPtr result = 0;
#else
   struct RESSTRUCT
   {
      int lw;
      int hw;
   } result = { 0, 0 };
   int rl, rh;
#endif
#ifndef __MSDOS__
 #ifndef NOASM
        BppInt savedsp;
        try
        {
           __asm
           {
              mov eax, esp
              mov savedsp, eax
              mov ecx, procargc
              mov edx, pargs
              cmp ecx, 0
              je docall
nxtarg:
              mov eax, [edx]
              push eax
              add edx, 4
              dec ecx
              jnz nxtarg
docall:
              call proc
              mov result, eax
              mov eax, savedsp
              mov esp, eax
           }
        }
        catch (...)
        {
        }
  #else
     BppIntPtr* p =  pargs;
     p++;
     result = (BppIntPtr) CallLibFuncCE(proc,argc-1,p);
  #endif
#else
   int savedsp;
   asm{
                     push es
                     push di
                     mov ax, sp
                     mov savedsp, ax
         mov cx, procargc
                        les di,pargs
         cmp cx, 0
         je docall
#ifdef __TURBOC__
      }
nxtarg:
     asm {
#else
nxtarg:
#endif
                        mov ax, es:[di]
                        add di, 2
                        mov bx, es:[di]
                        add di, 2
                        push bx
         push ax
         dec cx
         jnz nxtarg
#ifdef __TURBOC__
    }
docall:
    asm {
#else
docall:
#endif
#ifdef __WATCOMC__
                        call dword ptr proc
#else
                        call dword ptr proc
#endif
                        mov rl, ax
                        mov rh, dx
                        mov ax, savedsp
                        mov sp, ax
                        pop di
                        pop es
                }
                result.lw = rl;
      result.hw = rh;
#endif // __MSDOS__
      freemem(pargs);
  *vp = *((BppIntPtr*) &result);
  sp = vp;
  return true;
}

#endif //!DOXY
