#include "dictio.h"
#include "bppprint.h"
#include "bppbuf.h"
#include "bppclass.h"
#include "bppvec.h"
#include "bppobjct.h"
#include "bppvalue.h"
#include "bppvm.h"
#include "bppconst.h"
#include "bppdfunc.h"
struct ObjListEntry
{
   ObjListEntry():ID(0),pval(0),next(0)   {}
   ~ObjListEntry()
   {
      if (next != 0)
      {
         delete next;
         next = 0;
      }
   }
   void* ID;
   BppValue val;
   BppValue* pval;
   ObjListEntry* next;
};

bool BppDictionaryIO::addOLEntry(void* ID, const BppValue& val)
{
	ObjListEntry* ep = _objList;
	while (ep != 0)
	{
		if (ep->ID == ID)
		   return false;
		ep = ep->next;
	}
	ep = new ObjListEntry();
	ep->ID = ID;
	ep->val=val;
	ep->next = _objList;
	_objList = ep;
	return true;
}

void BppDictionaryIO::addORefEntry(void* ID, BppValue* pval)
{
	ObjListEntry* ep = new ObjListEntry();
	ep->ID = ID;
	ep->pval = pval;
	ep->next = _objRefList;
	_objRefList = ep;
}

struct VarListEntry
{
   VarListEntry() : var(0), key(0),dictname(0),next(0) {}
   ~VarListEntry()
   {
      if (key != 0) key->release();
      if (dictname != 0) dictname->release();
      if (next != 0)
      {
         delete next;
         next = 0;
      }
   }
   BppValue* var;
   BppString* key;
   BppString* dictname;
   VarListEntry* next;
};

void BppDictionaryIO::clearObjectLists()
{
   if (_objList != 0)
      delete _objList;
   if (_objRefList != 0)
      delete _objRefList;
   if (_varList != 0)
      delete _varList;
   _objList = _objRefList = 0;
   _varList = 0;
}

BppDictionaryIO::BppDictionaryIO(BppVirtualMachine& vm)
:_classes(vm.classes()),_symbols(vm.symbols()),_print(vm.printer()),_vm(vm),_objList(0),_objRefList(0),_varList(0)
{
   _classes.addRef();
   _symbols.addRef();
}

BppDictionaryIO::~BppDictionaryIO()
{
   clearObjectLists();
   _classes.release();
   _symbols.release();
}

bool BppDictionaryIO::writeModule(const BppString& filename)
{

   FILE* fp = _tfopen(filename.c_str(),_T("wb"));
	if (fp == 0)
   {
      _print(_T("opening %s for write failed\n"),filename.c_str());
      return false;
   }

   bool result  = writeModule(fp);
   fclose(fp);
	return result;
}

bool BppDictionaryIO::writeModule(FILE* fp)
{
	if (fp == 0)
   {
      return false;
   }
   bool result  = writeDictionary(fp,_vm.modules());
   if (result) {
     // this->_vm.printer()(_T(" write classes\n"));
      result = writeDictionary(fp, _classes, true);
   }
   if (result) {
     // this->_vm.printer()(_T(" write symbols\n"));

		   result = writeDictionary(fp,_symbols,true);
   }
	clearObjectLists();
	return result;
}

bool BppDictionaryIO::writeSingleValue(FILE* fp, BppValue& value)
{
   bool ok = (fp != 0);
   clearObjectLists();
   if (ok)
   {
      ok = writeValue(fp,value);
      clearObjectLists();
   }
   return ok;
}


// writes
// //- string: name of owner class (empty if none)
//   - byte: sorted flag (1 = sorted, 0= unsorted)
//   - all entries
//   - empty string as end marker
bool BppDictionaryIO::writeDictionary(FILE* fp, BppDictionary& dict, bool ignoreNativeFuncs)
{
   BppByte sorted = dict._sorted ? 1 : 0;
   bool ok = writeBytes(fp,&sorted,sizeof(BppByte));
   for (BppDictionaryEntry* pe=dict.getFirst();ok && (pe!= 0) ;pe=pe->getNext())
   {
      ok = writeDictEntry(fp,*pe,ignoreNativeFuncs);
   }
   if (ok) ok = writeString(fp,_T("")); /* write empty string as end sign */
   return ok;
}

// writes
// - string: key
// - BppSymbolType: symbol type
// - value
bool BppDictionaryIO::writeDictEntry(FILE* fp, BppDictionaryEntry& entry,bool ignoreNativeFuncs)
{
   BppSymbolType st = entry.getSymbolType();
   const BppValue&  v = entry.getValue();
   BppValueType vtype = v.getType();
   if (ignoreNativeFuncs)
   {
      if ((vtype == BVT_CODE)/* && (st == ST_SFUNCTION)*/)
         return true;
      if ((st == ST_SDATA) && (vtype == BVT_FILE))
      {
        if (_tcsstr(_T("stdin;stdout;stderr;__OSTYPE__;__UNICODE__"),entry.getKey()->c_str()) != 0)
           return true;
      }
   }
   // always ignore native classes
   BppClass* pclass = v;
   if ((pclass != 0) && pclass->getIsNative())
      return true;
   // write key
   //if (pclass) 
   //   this->_vm.printer()(_T(" write dictkey: %s::%s\n"), pclass->getClassName()->c_str(),entry.getKey()->c_str());
   //else this->_vm.printer()(_T(" write dictkey: %s\n"), entry.getKey()->c_str());

   bool result = writeString(fp,entry.getKey()->c_str());
   // write symbol type
	if (result) result = writeBytes(fp,&st,sizeof(BppSymbolType));
   // write value
	if (result) result = writeValue(fp,(BppValue&)v);
   if (!result)
      _print(_T("error while writing dictionary entry %s\n"),entry.getKey()->c_str());
   return result;
}

// writes:
// - BppValueType: value type
// switch type
//   BVT_CLASS(REF):
//    - string: classname
//    if !writeref && not already written
//      - string: baseclass name (or "")
//      - writeDictionary(dataMembers)
//      - writeDictionary(funcMembers)
//      - BppInt: datasize
//    else
//      - string: "$" (end mark)
//   BVT_OBJECT(REF):
//    - pointer: to obj, used as ID
//    if not already written:
//      - string: classname
//      - write all member data entries (writeValue(val,writeClassRef=true))
//    else
//      - string: "$" (end mark)
//   BVT_BUFFER(REF):
//    - pointer: to buf, used as ID
//    if not already written:
//      - BppUInt: size (number of bytes)
//      - byte[size]: data
//    else
//      - BppInt: -1 (end mark)
//   BVT_CHARBUFFER(REF):
//    - pointer: to buf, used as ID
//    if not already written:
//      - BppUInt: size (number of characters)
//      - byte: chsize (size of character)
//      - byte[size * chsize]: data
//    else
//      - BppInt: -1 (end mark)
//   BVT_STRING(REF):
//    - pointer: to string, used as ID
//    if not already written:
//      - string: string content
//    else
//      - byte: 0xFF (end mark)
//   BVT_VECTOR(REF), BVT_BYTECODE(REF), BVT_LOCALVARINFO
//    - pointer: to vec, used as ID
//    if not already written:
//      - BppUInt: number of elements
//      - write all elements (writeValue(val,writeClassRef=true))
//    else
//      - BppInt: -1 (end mark)
//   BVT_CODE:
//      - string: name of native function
//   BVT_FILE:
//      - string: name of default file symbol
//   BVT_VAR:
//      - string: name of dictionary (or empty string on unnamed dictionary)
//      //- pointer: to dictionary, used as ID
//      - string: key (name of variable)
//   BVT_DICTIONARY(REF):
//      - pointer: to dict, used as ID
//      if not already written:
//         - writeDictionary(...)
//      else:
//         - byte: 0xFF as end marker
//   BVT_(all integral types):
//      - BppInt value of type specific size (1 .. 8 bytes)
//   BVT_FLOAT:
//      - float: float value
//   BVT_DOUBLE:
//      - double: double value
bool BppDictionaryIO::writeValue(FILE* fp, BppValue& value, bool writeClassref)
{
   static const BppUInt nullU = 0;
   BppValueType vtype = value.getType();
  unsigned char vflags = value.getFlags();
	writeBytes(fp,&vtype,1);
	writeBytes(fp,&vflags,1);
   bool ok = true;
	switch (vtype)
	{
		case BVT_CLASS:
      case BVT_CLASSREF:
         {
            BppClass* pclass = value;
            if (pclass == 0) return false;
            ok = writeString(fp,pclass->getClassName()->c_str());
            if ((!writeClassref) && ok && addOLEntry(pclass,value))
            {
               BppClass* pbase = pclass->getBaseClass();
               if (pbase == 0)
                  ok = writeString(fp,_T(""));
               else
                  ok = writeString(fp,pbase->getClassName()->c_str());
			   if (ok) ok=writeDictionary(fp,pclass->dataMembers());
			   if (ok) ok = writeDictionary(fp,pclass->funcMembers());
               BppInt size = pclass->getDataSize();
               if (ok) ok= writeBytes(fp,&size,sizeof(BppInt));
			}
            else
               ok = writeString(fp,_T("$"));
         }
		   break;
		case BVT_OBJECT:
      case BVT_OBJECTREF:
         {

            BppObject* pobj = value;
            // use current address of object as ID
			ok = writePtr(fp, (BppIntPtr)pobj); /* ID */
            if (pobj == 0) return true;
            if (ok)
            {
               if (addOLEntry(pobj,value))
               {
                  pobj->getClass()->notifyObjSerialization(pobj, true);
                  ok = writeString(fp,pobj->getClass()->getClassName()->c_str());
                  BppVector* mdata = pobj->getMemberData();
                  BppUInt n = mdata->size()-1;
				  ok = ok && writeBytes(fp, &n, sizeof(BppUInt));
				  for (BppUInt i=1; ok && (i<=n); i++)
                     ok = writeValue(fp,(*mdata)[i],true);
               }
               else
                  ok = writeString(fp,_T("$"));
            }
         }
		   break;
      case BVT_BUFFER:
      case BVT_BUFFERREF:
         {
            BppBuffer* pbuf = value;

			ok = writePtr(fp, (BppIntPtr)pbuf); /* ID */
            if (pbuf == 0) return true;
            if (ok)
            {
               if (addOLEntry(pbuf,value))
               {
                  BppUInt size = pbuf->size();
                  ok = writeBytes(fp,&size,sizeof(BppUInt));
                  if (ok) ok = writeBytes(fp,pbuf->data(),size);
               }
               else
               {
                  BppInt endmark = -1;
                  ok = writeBytes(fp,&endmark,sizeof(BppInt));
               }
            }
         }
         break;
      case BVT_CHARBUFFER:
      case BVT_CHARBUFFERREF:
         {
            BppCharBuffer* pbuf = value;

			ok = writePtr(fp, (BppIntPtr)pbuf); /* ID */
            if (pbuf == 0) return true;
            if (ok)
            {
               if (addOLEntry(pbuf,value))
               {
                  BppUInt size = pbuf->size();
                  ok = writeBytes(fp,&size,sizeof(BppUInt));
                  BppByte chsize = sizeof(BppChar);
                  if (ok) ok = writeBytes(fp,&chsize,1); // write character size
                  if (ok) ok = writeBytes(fp,pbuf->data(),size * chsize);
               }
               else
               {
                  BppInt endmark = -1;
                  ok = writeBytes(fp,&endmark,sizeof(BppInt));
               }
            }
         }
         break;
		case BVT_STRING:
      case BVT_STRINGREF:
         {
            BppString* pbuf = value;

			ok = writePtr(fp, (BppIntPtr)pbuf); /* ID */
            if (pbuf == 0) return true;
            if (ok)
            {
               if (addOLEntry(pbuf,value))
               {
                  ok = writeString(fp,pbuf->c_str());
               }
               else
               {
                  BppByte endmark = 0xFF;
                  ok = writeBytes(fp,&endmark,sizeof(BppByte));
               }
            }
         }
		   break;
		case BVT_BYTECODE:
      case BVT_BYTECODEREF:
      case BVT_LOCALVARINFO:
     	case BVT_VECTOR:
      case BVT_VECTORREF:
      case BVT_STRONGOBJECT:
         {
            BppVector* pvec = vtype == BVT_STRONGOBJECT ? value.getStrongObjDesc() : value;

			ok = writePtr(fp, (BppIntPtr)pvec); /* ID */
            if (pvec == 0) return true;
            if (ok)
            {
               if (addOLEntry(pvec,value))
               {
                  BppUInt size = pvec->size();
                  ok = writeBytes(fp,&size,sizeof(BppUInt));
                  for (BppUInt i=0; ok && (i<size); i++)
                  {
                     ok = writeValue(fp,(*pvec)[i],true);
                  }
               }
               else
               {
                  BppInt endmark = -1;
                  ok = writeBytes(fp,&endmark,sizeof(BppInt));
               }
            }
         }
		   break;
      case BVT_DICTIONARY:
      case BVT_DICTIONARYREF:
         {
            BppDictionary* pdict = value;
			ok = writePtr(fp, (BppIntPtr)pdict); /* ID */
            if (pdict == 0)
               return true;
            if (ok)
            {
               if (addOLEntry(pdict,value))
               {
                  ok = writeDictionary(fp,*pdict);
               }
               else
               {
                  BppByte endmark = 0xFF;
                  ok = writeBytes(fp,&endmark,sizeof(BppByte));
               }
            }
         }
         break;
		case BVT_CODE:
         {
            ok = writeString(fp,findSymbolNameFromFuncPtr((void*)(BPPUSERFUNC)value));
         }
         break;
		case BVT_FILE:
         {
            ok = writeString(fp,findSymbolNameFromFuncPtr((FILE*)value));
         }
         break;
		case BVT_VAR: // reference to variable
         {
            BppDictionaryEntry* pvar = value;
            if (pvar == 0) return false;
            BppDictionary* pdict = pvar->getDictionary();
            ok = writeString(fp,getDictionaryName(pdict)); // name of static dictionary or empty string
            if (ok) ok = writeString(fp,pvar->getKey()->c_str());
         }
			break;
      case BVT_INT:
         {
            BppInt v = value;
            ok = writeBytes(fp,&v,sizeof(BppInt));
         }
         break;
      case BVT_UINT:
         {
            BppUInt v = value;
            ok = writeBytes(fp,&v,sizeof(BppUInt));
         }
         break;
      case BVT_SHORT:
         {
            BppShort v = value;
            ok = writeBytes(fp,&v,sizeof(BppShort));
         }
         break;
      case BVT_USHORT:
         {
            BppUShort v = value;
            ok = writeBytes(fp,&v,sizeof(BppUShort));
         }
         break;
      case BVT_SBYTE:
         {
            BppSByte v = value;
            ok = writeBytes(fp,&v,sizeof(BppSByte));
         }
         break;
      case BVT_BYTE:
         {
            BppByte v = value;
            ok = writeBytes(fp,&v,sizeof(BppByte));
         }
         break;
      case BVT_LONG:
         {
            BppLong v = value;
            ok = writeBytes(fp,&v,sizeof(BppLong));
         }
         break;
      case BVT_ULONG:
         {
            BppULong v = value;
            ok = writeBytes(fp,&v,sizeof(BppULong));
         }
         break;
      case BVT_INTPTR:
         {
            BppIntPtr v = value;
			ok = writePtr(fp,v); /* ID */
         }
         break;
      case BVT_FLOAT:
         {
            float v = value;
            ok = writeBytes(fp,&v,sizeof(float));
         }
         break;
      case BVT_DOUBLE:
         {
            double v = value;
            ok = writeBytes(fp,&v,sizeof(double));
         }
         break;
	}
	return ok;
}


// writes
// - byte: csize (size of character)
// - byte[(len+1) * csize]: string content
bool BppDictionaryIO::writeString(FILE* fp, const BppChar* str)
{
   // write one byte containing character size of string
   BppByte csize = sizeof(BppChar);
   fwrite(&csize,1,1,fp);
   if (str == 0)
   {
      // null makes an empty string
      BppChar c = 0;
      return (fwrite(&c,csize,1,fp) == 1);
   }
   // write string including null character on end
   size_t len = _tcslen(str)+1;
   return (fwrite(str,csize,len,fp) == len);
}

bool BppDictionaryIO::writePtr(FILE * fp, BppIntPtr ptr)
{
	BppByte psize = sizeof(BppIntPtr);
	fwrite(&psize, 1, 1, fp);
	return fwrite(&ptr, psize, 1, fp) == 1;
}

// writes
// - byte[cnt]: cnt bytes of data
bool BppDictionaryIO::writeBytes(FILE* fp, const void* src, BppUInt cnt)
{
   return (cnt == 0 || fwrite(src,cnt,1,fp) == 1);
}

bool BppDictionaryIO::readModule(const BppString& filename)
{

   FILE* fp = _tfopen(filename.c_str(),_T("rb"));
	if (fp == 0)
   {
      _print(_T("opening %s for read failed\n"),filename.c_str());
      return false;
   }

   bool result = readModule(fp);
   fclose(fp);
	return result;
}

bool BppDictionaryIO::readModule(FILE* fp)
{

	if (fp == 0)
   {
      return false;
   }
   BppDictionary tmpModules;
   bool result = readDictionary(fp,tmpModules,true);
   for (BppDictionaryEntry* pe = tmpModules.getFirst();pe != 0;pe=pe->getNext())
   {
      if (!_vm.loadNativeModule(pe->getKey()->c_str()))
      {
         result = false;
         break;
      }
   }
   if (result)
   {
      //this->_vm.printer()(_T(" read classes\n"));

      result = readDictionary(fp,_classes,true);
      if (result) {
         for (BppDictionaryEntry* p = _classes.getFirst(); p; p = p->getNext())
         {
            BppClass* pc = p->getValue();
            if (pc) {
               pc->buildDefaultValues();
               pc->computeDataSize();
            }
         }
      }
   }
   if (result) {
     // this->_vm.printer()(_T(" read symbols\n"));
      result = readDictionary(fp, _symbols, true);
   }
   if (result)
   {
      BppDefaultFunctions::registerSysVariables(&_vm);
      BppDefaultFunctions::registerFiles(&_vm);
   }
	dissolveVars();
	dissolveObjRefs();
	clearObjectLists();
	return result;
}

bool BppDictionaryIO::readSingleValue(FILE* fp, BppValue& value)
{
   clearObjectLists();
   bool ok = (fp != 0);
   if (ok)
   {
      ok = readValue(fp,value);
      dissolveVars();
      dissolveObjRefs();
      clearObjectLists();
   }
   return ok;
}

bool BppDictionaryIO::readDictionary(FILE* fp, BppDictionary& dict, bool readSortedFlag)
{
   if (readSortedFlag)
   {
      BppByte b;
      bool ok = readBytes(fp,&b,sizeof(BppByte));
      if (!ok || b > 1)
      {
#ifdef _DEBUG
         fpos_t pos;
         fgetpos(fp, &pos);
         _print(_T("error reading dictionary (sorted flag) at position %u\n"),(unsigned) pos);
#endif
         return false;
      }
      dict._sorted = (b == 1);
   }
	while (readDictEntry(fp, dict));
   return true;
}

bool BppDictionaryIO::readDictEntry(FILE* fp, BppDictionary& dict)
{
    BppString* key = readString(fp);
    if ((key == 0) || (key->length() == 0))
       return false;
    key->addRef();
	//this->_vm.printer()(_T(" read dictkey: %s\n"), key->c_str());
    BppDictionaryEntry* entry = dict[key];
    key->release();
    BppSymbolType stype;
    if (!readBytes(fp,&stype,sizeof(BppSymbolType)))
       return false;
    entry->setSymbolType(stype);
    if (!readValue(fp,entry->_value))
       return false;
    return true;
}


bool BppDictionaryIO::readValue(FILE* fp, BppValue& value, bool readClassref)
{
   static BppUInt nullUBuf;
   unsigned char bytebuf;
   BppValueType vtype;
   if (!readBytes(fp,&bytebuf,1))
      return false;
   vtype = (BppValueType) bytebuf;
   //value.dispose();
   if (!readBytes(fp,&bytebuf,1))
      return false;
   bool ok = true;
	switch (vtype)
	{
		case BVT_CLASS:
      case BVT_CLASSREF:
         {
            BppString* className = readString(fp);
            if (className == 0) 
				return false;
            className->addRef();

            BppString* baseName = readString(fp);
            if (baseName == 0)
            {
               className->release();
               return false;
            }
            baseName->addRef();
            if ((!readClassref) && (_tcscmp(baseName->c_str(),_T("$")) != 0))
            {
               BppClass* base = 0;
               if (baseName->length() > 0)
               {
                  BppDictionaryEntry* entry = _classes.find(*baseName);
                  if (entry != 0)
                     base = entry->getValue();
				  else {
					  base = new BppClass(_vm, baseName);
					  _classes[*baseName] = base;
				  }
			   }
               baseName->release();
			   BppClass* newClass = 0;
			   BppDictionaryEntry* centry = _classes.find(*className);
			   if (centry) {
				   newClass = centry->getValue();
				   if (newClass) {
					   BppClass* curbase = newClass->getBaseClass();
					   if (base && (!curbase)) newClass->setBaseClass(base);
					   else if (base != curbase) {
#ifdef _DEBUG
						   _print(_T("illegal redefinition of base class for class %s\n"), className->c_str());
#endif
                     className->release();
						   return false;
					   }
				   }
			   }
			   if (!newClass) {
				   newClass = new BppClass(_vm, className, base);
               centry = _classes[className];
               centry->setValue(newClass);               
			   }
				   value = BppValue(newClass, (bool)(vtype == BVT_CLASSREF));
				   if (centry) centry->setSymbolType(ST_CLASS);
               className->release();
               if (! (readDictionary(fp,newClass->dataMembers(),true) &&
                      readDictionary(fp,newClass->funcMembers(),true)) )
                      return false;
			   BppInt sz = 0;
               if (!readBytes(fp,&sz,sizeof(BppInt)))
                  return false;
			   newClass->_size = sz;
            }
            else
            {
               baseName->release();
               BppDictionaryEntry* entry = _classes.find(*className);
			   if ((!entry) || (!(BppClass*)entry->getValue())) {
				   _classes[*className] = new BppClass(_vm, className);
				   entry = _classes.find(*className);
				   entry->setSymbolType(ST_CLASS);
			   }
               ok = (entry != 0);
               if (ok)
                  value = BppValue((BppClass*)entry->getValue(),(bool)(vtype == BVT_CLASSREF));
               //else
               //   _print(_T("class %s not found\n"),className->c_str());
               className->release();
            }
         }
		   break;
		case BVT_OBJECT:
      case BVT_OBJECTREF:
         {
			 void* id = (void*)readPtr(fp, ok);/* ID */
            if (!ok) break;
			if (id == 0)
			{
				value = BppValue((BppRefObject*)0,vtype);
				break;
			}
            BppString* className = readString(fp); /* classname */
            if (className == 0) 
				return false;
            className->addRef();
            if ((className->length() > 0) && ((*className)[0] != _T('$')))
            {
               BppDictionaryEntry* entry = _classes.find(*className);
               BppClass* pclass = (entry != 0) ? (BppClass*)entry->getValue() : (BppClass*)0;
               if (!pclass) {
                  _classes[*className] = new BppClass(_vm, className);
                  entry = _classes.find(*className);
                  entry->setSymbolType(ST_CLASS);
                  pclass = (entry != 0) ? (BppClass*)entry->getValue() : (BppClass*)0;
               }
               if (pclass != 0)
               {
                  BppObject* pobj = new BppObject(pclass);
                  value = BppValue(pobj,(bool)(vtype == BVT_OBJECTREF));
                  addOLEntry(id,value);
                  BppVector* mdata = pobj->getMemberData();
                  BppUInt n = mdata->size();
				  BppUInt nr = 0;

				  ok = ok && readBytes(fp, &nr, sizeof(BppUInt));
				  if (ok && nr+1 > n) {
					  mdata->insert(-1, BppValue(), (nr+1) - n);
					  n = nr+1;
				  }
				  (*mdata)[0] = pclass;
                  for (BppUInt i=1; ok && (i<n); i++)
                     ok = readValue(fp,(*mdata)[i],true);
                  if (ok) pclass->notifyObjSerialization(pobj, false);
               }
			   else {
				   //_print(_T("class %s not found\n"), className->c_str());
				   ok = false;
			   }
            }
            else
            {
               value = BppValue((BppRefObject*)0,vtype);
               addORefEntry(id,&value);
            }
            className->release();
         }
         break;
		case BVT_BUFFER:
      case BVT_BUFFERREF:
         {
			 void* id = (void*)readPtr(fp, ok);/* ID */
            if (!ok) break;
			if (id == 0)
			{
				value = BppValue((BppRefObject*)0,vtype);
				break;
			}
            BppUInt size = 0;
            ok = readBytes(fp,&size,sizeof(BppUInt));
            if (!ok) break;
            if (((BppInt)size) != -1) // no end mark
            {
               BppBuffer* pbuf = new BppBuffer(size);
               value = BppValue(pbuf,(bool)(vtype == BVT_BUFFERREF));
               addOLEntry(id,value);
               pbuf->_size = size;
               ok = readBytes(fp,pbuf->_data,size);
            }
            else
            {
               value = BppValue((BppRefObject*)0,vtype);
               addORefEntry(id,&value);
            }
         }
         break;
		case BVT_CHARBUFFER:
      case BVT_CHARBUFFERREF:
         {
			 void* id = (void*)readPtr(fp, ok);/* ID */
			 if (!ok)
				break;
			if (id == 0)
			{
				value = BppValue((BppRefObject*)0,vtype);
				break;
			}
            BppUInt size = 0;
            ok = readBytes(fp,&size,sizeof(BppUInt));
            if (!ok) 
				break;
            if (((BppInt)size) != -1) // no end mark
            {
               BppByte chsize = 0;
               ok = readBytes(fp,&chsize,sizeof(BppByte));
               if (!ok) 
				   break;
               BppCharBuffer* pbuf = new BppCharBuffer(size);
               value = BppValue(pbuf,(bool)(vtype == BVT_CHARBUFFERREF));
               addOLEntry(id,value);
               BppInt cbuf = 0;
               for (BppUInt i=0; ok && (i<size) ; i++)
               {
                  ok = readBytes(fp,&cbuf,chsize);
                  pbuf->push_back((BppChar) cbuf);
               }
            }
            else
            {
               value = BppValue((BppRefObject*)0,vtype);
               addORefEntry(id,&value);
            }
         }
         break;
		case BVT_STRING:
      case BVT_STRINGREF:
         {
			 void* id = (void*)readPtr(fp, ok);/* ID */
			 if (!ok)
				break;
			if (id == 0)
			{
				value = BppValue((BppRefObject*)0,vtype);
				break;
			}
            BppString* pstr = readString(fp);
            value = BppValue(pstr,(bool)(vtype == BVT_STRINGREF));
            if (pstr != 0)
               addOLEntry(id,value);
            else
               addORefEntry(id,&value);
         }
		   break;
		case BVT_BYTECODE:
      case BVT_BYTECODEREF:
      case BVT_LOCALVARINFO:
     	case BVT_VECTOR:
      case BVT_VECTORREF:
      case BVT_STRONGOBJECT:
      {
            bool isref = (vtype >= _BVTREFMIN);
            bool isstrongobj = (vtype == BVT_STRONGOBJECT);
            bool iscode = ((vtype == BVT_BYTECODE) || (vtype == BVT_BYTECODEREF)|| (vtype == BVT_LOCALVARINFO));
			void* id = (void*)readPtr(fp, ok);/* ID */
			if (id == 0)
			{
				value = BppValue((BppRefObject*)0,vtype);
				break;
			}
            if (!ok) 
				break;
            BppUInt size = 0;
            ok = readBytes(fp,&size,sizeof(BppUInt));
            if (!ok) 
				break;
            if (((BppInt)size) != -1) // no end mark
            {
               BppVector* pvec = new BppVector(size,true);
               if (isstrongobj)
               {
                  value = BppValue(vtype, BVF_STRONGTYPED,false);
                  value.assignRef(pvec);
               }
               else
                  value = BppValue(pvec,iscode,isref);
               addOLEntry(id,value);
               for (BppUInt i=0; ok && (i<size); i++)
               {
                  ok = readValue(fp,(*pvec)[i],true);
               }
            }
            else
            {
               value = BppValue((BppRefObject*)0,vtype);
               addORefEntry(id,&value);
            }
         }
		   break;
      case BVT_DICTIONARY:
      case BVT_DICTIONARYREF:
         {
			 void* id = (void*)readPtr(fp, ok);/* ID */
			 if (!ok)
				break;
			if (id == 0)
			{
				value = BppValue((BppRefObject*)0,vtype);
				break;
			}
            BppByte sortflag = 0;
            ok = readBytes(fp,&sortflag,sizeof(BppByte));
            if (!ok) 
				break;
            if (sortflag != 0xFF) // no end mark
            {
               BppDictionary* pdict = new BppDictionary((bool)(sortflag == 1));
					value = BppValue(pdict,(bool)(vtype == BVT_DICTIONARYREF));
               addOLEntry(id,value);
               ok = readDictionary(fp,*pdict,false);
            }
            else
            {
               value = BppValue((BppRefObject*)0,vtype);
               addORefEntry(id,&value);
            }
         }
         break;
		case BVT_CODE:
      case BVT_FILE:
         {
            BppString* pname = readString(fp);
            if (pname == 0) 
				return false;
            pname->addRef();
            if (pname->length() == 0)
            {
               pname->release();
               value = BppValue((BppRefObject*)0,vtype);
               break;
            }
            BppDictionaryEntry* entry = _symbols.find(*pname);
            pname->release();
            if (entry == 0) 
				return false;
            value = entry->getValue();
         }
         break;
		case BVT_VAR:
		    ok = readVarListEntry(fp,value);
			break;
		case BVT_INT:
         {
            BppInt v = 0;
            ok = readBytes(fp,&v,sizeof(BppInt));
            if (ok) value = BppValue(v);
         }
         break;
		case BVT_UINT:
         {
            BppUInt v = 0;
            ok = readBytes(fp,&v,sizeof(BppUInt));
            if (ok) value = BppValue(v);
         }
         break;
		case BVT_SHORT:
         {
            BppShort v = 0;
            ok = readBytes(fp,&v,sizeof(BppShort));
            if (ok) value = BppValue(v);
         }
         break;
		case BVT_USHORT:
         {
            BppUShort v = 0;
            ok = readBytes(fp,&v,sizeof(BppUShort));
            if (ok) value = BppValue(v);
         }
         break;
		case BVT_SBYTE:
         {
            BppShort v = 0;
            ok = readBytes(fp,&v,sizeof(BppSByte));
            if (ok) value = BppValue(v);
         }
         break;
		case BVT_BYTE:
         {
            BppByte v = 0;
            ok = readBytes(fp,&v,sizeof(BppByte));
            if (ok) value = BppValue(v);
         }
         break;
		case BVT_LONG:
         {
            BppLong v = 0;
            ok = readBytes(fp,&v,sizeof(BppLong));
            if (ok) value = BppValue(v);
         }
         break;
		case BVT_ULONG:
         {
            BppULong v = 0L;
            ok = readBytes(fp,&v,sizeof(BppULong));
            if (ok) value = BppValue(v);
         }
         break;
		case BVT_INTPTR:
         {
            BppIntPtr v = readPtr(fp,ok);
            if (ok) value = BppValue(v,true);
         }
         break;
		case BVT_FLOAT:
         {
            float v = 0;
            ok = readBytes(fp,&v,sizeof(float));
            if (ok)
               value = BppValue(v);
         }
         break;
		case BVT_DOUBLE:
         {
            double v = 0;
            ok = readBytes(fp,&v,sizeof(double));
            if (ok)
               value = BppValue(v);
         }
         break;
      default:
         value.dispose();
	}
   value.setFlags(bytebuf);
	return ok;
}

bool BppDictionaryIO::readVarListEntry(FILE* fp, BppValue& val)
{
   BppString* dictname = readString(fp);
   BppString* key = readString(fp);
   bool ok = ((dictname != 0) && (key != 0));
   if (ok)
   {
      VarListEntry* ve = new VarListEntry();
      ve->next = _varList;
      _varList = ve;
      dictname->addRef();
      ve->dictname = dictname;
      key->addRef();
      ve->key = key;
      ve->var = &val;
   }
   else
   {
      if (dictname != 0) delete dictname;
      if (key != 0) delete key;
   }
   return ok;
}

BppString* BppDictionaryIO::readString(FILE* fp)
{
   BppByte csize = 0;
   if (fread(&csize,1,1,fp) != 1)
      return 0;
   if (csize == 0xFF) return 0;  // flag for reference only
//      char bbuf[100];
   if (csize == 0 || csize > 2)
   {
#ifdef _DEBUG
      fpos_t pos;
      fgetpos(fp, &pos);
      _print(_T("error reading string at position %u\n"), (unsigned)pos);
#endif
       return 0; //fread(bbuf,80,1,fp);
   }
   BppCharBuffer buf(32); // default buffer size
   BppInt cbuf = 0;
   while (!feof(fp))
   {
       fread(&cbuf,csize,1,fp);
       buf.push_back((BppChar) cbuf);
       if (cbuf == 0)
          break;
   }
   return new BppString((const BppChar*)buf.data());
}

BppIntPtr BppDictionaryIO::readPtr(FILE * fp, bool& ok)
{
	union {
		BppIntPtr p;
		char s[8];
	} buf;
	buf.p = 0;
	ok = fread(&buf, 1, 1, fp) == 1;
	if (ok) {
		size_t siz = (size_t)buf.s[0];
		ok = (siz == 4 || siz == 8);
		if (ok) {
			ok = fread(&buf, siz, 1, fp) == 1;
			if (ok) return buf.p;
		}
	}
	return 0;
}


bool BppDictionaryIO::readBytes(FILE* fp, void* pbuf, BppUInt cnt)
{
   return (cnt == 0 || fread(pbuf,cnt,1,fp) == 1);
}


void BppDictionaryIO::dissolveVars()
{
   while (_varList != 0)
   {
      VarListEntry* vp = _varList;
      BppDictionary*  dict = findDictionaryByName(vp->dictname->c_str());
      if (dict != 0)
      {
         BppDictionaryEntry* entry = dict->find(*(vp->key));
         if (entry != 0)
            *(vp->var) = BppValue(entry);
      }
      _varList = vp->next;
      vp->next = 0;
      delete vp;
   }
}

void BppDictionaryIO::dissolveObjRefs()
{
   for (ObjListEntry* po = _objList; po != 0; po=po->next)
   {
      for (ObjListEntry* pr = _objRefList; pr != 0; pr = pr->next)
      {
         if (pr->ID == po->ID)
         {
            if (pr->pval != 0)
               pr->pval->assignRef(po->val.getRef());
         }
      }
   }
   clearObjectLists();
}

const BppChar* BppDictionaryIO::findSymbolNameFromFuncPtr(void* ptr)
{
   if (ptr == 0) return 0;
   for (BppDictionaryEntry* pe=_symbols.getFirst();pe != 0;pe = pe->getNext())
   {
      const BppValue& val = pe->getValue();
      if ((ptr == (BPPUSERFUNC) val) || (ptr == (FILE*) val))
         return pe->getKey()->c_str();
   }
   return 0;
}

#define BPPSYMDICT _T("!S")
#define BPPCLSDICT _T("!C")
#define BPPDATAPREFIX _T("m!")
#define BPPFUNCPREFIX _T("f!")
static const size_t BPPPREFIXLENGTH = _tcslen(BPPDATAPREFIX);

const BppChar* BppDictionaryIO::getDictionaryName(const BppDictionary* dict) const
{
   static BppString s;
   if (dict != 0)
   {
      if (dict == &_classes) return BPPCLSDICT;
      if (dict == &_symbols) return BPPSYMDICT;
      BppClass* pclass = dict->_ownerClass;
      if (pclass != 0)
      {
         if (dict == pclass->_dataMembers)
            s = BPPDATAPREFIX;
         else
            s = BPPFUNCPREFIX;
         s += *pclass->getClassName();
         return s.c_str();
      }
   }
   return 0;
}

BppDictionary* BppDictionaryIO::findDictionaryByName(const BppChar* name)
{
   if (name != 0)
   {
      if (_tcscmp(name,BPPSYMDICT) == 0) return &_symbols;
      if (_tcscmp(name,BPPCLSDICT) == 0) return &_classes;
      bool isdata = true;
      if (_tcsstr(name, BPPDATAPREFIX) != name) // no data member
      {
         isdata = false;
         if (_tcsstr(name, BPPFUNCPREFIX) != name) // no function member
            return 0;
      }
      BppDictionaryEntry* pclassentry = _classes.find(name+BPPPREFIXLENGTH);
      if (pclassentry != 0)
      {
         BppClass* pclass = pclassentry->getValue();
         if (pclass != 0)
            return (isdata) ? pclass->_dataMembers : pclass->_funcMembers;
      }
   }
   return 0;
}
