#include "int64.h"
#include <math.h>
#include <stdlib.h>
#include <string.h>
Int64::Int64()
:_high(0),_low(0)
{
}

Int64::Int64(long high, unsigned long low)
:_high(high),_low(low)
{
}

Int64::Int64(const Int64& val)
:_high(val._high),_low(val._low)
{
}

Int64::Int64(long val)
:_high(val<0 ? -1 : 0),_low((unsigned long) val)
{
}

Int64::Int64(unsigned long val)
:_high(0),_low(val)
{
}

Int64::Int64(int val)
:_high(val<0 ? -1 : 0),_low((unsigned int) val)
{
}

Int64::Int64(unsigned int val)
:_high(0),_low(val)
{
}

Int64::Int64(short val)
:_high(val < 0 ? -1 : 0),_low((unsigned long)(int) val)
{
}

Int64::Int64(unsigned short val)
:_high(0),_low(val)
{
}

Int64::Int64(float val)
:_high(0),_low(0)
{
   static const float divisor = (float)(unsigned)0xffffffff + 1.0;
   bool neg = (val < 0);
   if (neg) val = -val;
   _low = (unsigned long) fmod(val,divisor);
   _high = (unsigned long) (val / divisor);
   if (neg) (*this) = -(*this);
}

Int64::Int64(double val)
:_high(0),_low(0)
{
   static const double divisor = (double)(unsigned)0xffffffff + 1.0;
   bool neg = (val < 0);
   if (neg) val = -val;
   _low = (unsigned long) fmod(val,divisor);
   _high = (unsigned long) (val / divisor);
   if (neg) (*this) = -(*this);
}

Int64::~Int64()
{
}

Int64& Int64::operator=(const Int64& val)
{
   if (&val != this)
   {
      _high = val._high;
      _low = val._low;
   }
   return *this;
}

Int64& Int64::operator++()
{
   unsigned long l = _low;
   if (++_low < l) ++_high;
   return *this;
}

const Int64 Int64::pinc()
{
   Int64 result = *this;
   ++(*this);
   return result;
}

Int64& Int64::operator--()
{
   unsigned long l = _low;
   if (--_low > l) --_high;
   return *this;
}
const Int64 Int64::pdec()
{
   Int64 result = *this;
   --(*this);
   return result;
}

Int64 Int64::operator~() const
{
	return Int64(~_high,~_low);
}

Int64 Int64::operator-() const
{
	return ++Int64(~_high,~_low);
}

int Int64::compare(const Int64& cmp) const
{
   if (_high < cmp._high) return -1;
   if (_high > cmp._high) return 1;
   if (_low == cmp._low) return 0;
   if (_high >= 0)
   {
      return (_low > cmp._low) ? 1 : -1;
   }
   return (_low < cmp._low) ? 1 : -1;
}

Int64& Int64::operator+=(const Int64& val)
{
   unsigned long l = _low;
   _low += val._low;
   if (_low < l) ++_high;
   _high += val._high;
   return *this;
}

Int64& Int64::operator-=(const Int64& val)
{
   (*this) += -val;
   return *this;
}

Int64& Int64::operator&=(const Int64& val)
{
   _high &= val._high;
   _low &= val._low;
   return *this;
}

Int64& Int64::operator|=(const Int64& val)
{
   _high |= val._high;
   _low |= val._low;
   return *this;
}

Int64& Int64::operator^=(const Int64& val)
{
   _high ^= val._high;
   _low ^= val._low;
   return *this;
}


Int64& Int64::operator<<=(unsigned nbits)
{
   nbits &= 0x3F;
   for (unsigned i=0;i<nbits;++i)
   {
      _high <<= 1;
      if (_low & 0x80000000L)
         _high |=1;
      _low <<= 1;
   }
   return *this;
}

Int64& Int64::operator>>=(unsigned nbits)
{
   nbits &= 0x3F;
   unsigned sign = _high & 0x80000000L;
   for (unsigned i=0;i<nbits;++i)
   {
      _low >>= 1;
      if (_high & 1)
         _low |= 0x80000000L;
      _high >>= 1;
      _high |= sign;
   }
   return *this;
}

Int64 Int64::operator<<(unsigned nbits) const
{
   return Int64(*this) <<= nbits;
}

Int64 Int64::operator>>(unsigned nbits) const
{
   return Int64(*this) >>= nbits;
}

Int64& Int64::operator*=(const Int64& val)
{
   if ((_high == 0) && (_low == 0))
     return *this;
   if ((val._high == 0) && (val._low == 0))
   {
      return (*this = val);
   }
   *this = (*this * val);
   //Int64 rop = val;
   //Int64 lop = *this;
   //int sign = 1;
   //if (_high < 0)
   //{
   //   lop = -(*this);
   //   sign *= -1;
   //}
   //if (rop._high < 0)
   //{
   //   rop = -rop;
   //   sign *= -1;
   //}
   //_high = _low = 0;
   //unsigned short* pl = (unsigned short*) &lop;
   //unsigned short* pr = (unsigned short*) &rop;
   //for (int i=0; i< 4;++i)
	  // for (int j=0;j<4;++j)
	  // {
   //        if (i == 3 && j == 3) break;
		 //  BppULong((long)pl[i] * (long)pr[j]) << 16*(i+j)
	  // }
   //Int64 mask = 1;
   //for (int i=0; i< 63;++i)
   //{
   //   if ((rop._low & mask._low) || (rop._high & mask._high))
   //   {
   //      *this += lop;
   //   }
   //   lop <<= 1;
   //   mask <<= 1;
   //}
   //if (sign < 0) *this = -(*this);
   return *this;
}

bool Int64::_idiv(const Int64& dividend, const Int64& divisor, Int64& q, Int64& r)
{
   if (!(divisor._high || divisor._low)) // divisor is zero
   {
      if (dividend._high < 0)
      {
         q._high = r._high = 0x80000000;
       }
      else 
      {
         r._high = 0x7fffffff;
         r._low = 0xffffffff;
         if (dividend._high || dividend._low) // dividend is not zero
         {
            q = r;
         }
      }
      return false;
   }
   if (!(dividend._high || dividend._low)) // dividend is zero
   {
	   q._high = r._high = 0;
	   q._low = r._low = 0;
      return true;
   }
   q._high = r._high = 0;
   q._low = r._low = 0;
   Int64 dend = (dividend._high < 0) ? -dividend : dividend;
   Int64 dor = (divisor._high < 0) ? -divisor : divisor;
   long nh = dividend._high;
   long dh = divisor._high;
  int c = dend.compare(dor);
   if (c == 0)
   {
      q._low = 1;
   }
   else if (c < 0)
   {
      r = dend;
   }
   else
   {
      Int64 qtmp, dd;;
      while (1)
      {
         dd = dor;
         qtmp = 1;
         while ((c = dend.compare(dd <<= 1)) >= 0)
            qtmp <<= 1;
         dd >>= 1; //qtmp >>=1;
         q += qtmp; dend -= dd;
         c = dend.compare(dor);
         if (c == 0)
         {
            ++q;
            break;
         }
         else if (c < 0)
         {
            r = /*dor -*/ dend;
            break;
         }      
      }
   }

  ///* the sign of the remainder is the same as the sign of the dividend
  //   and the quotient is negated if the signs of the operands are
  //   opposite */
  //quotient = q;
   if (nh < 0) {
    r = -r;
    if (dh > 0)
      q = -q;
  }
  else { /* positive dividend */
     if (dh < 0)
       q = -q;
  }
  
   return true;
}

Int64& Int64::operator/=(const Int64& val)
{
   Int64 q,r;
   _idiv(*this, val, q, r);
   *this = q;
   return *this;
}

Int64& Int64::operator%=(const Int64& val)
{
   Int64 q,r;
   _idiv(*this, val, q, r);
   *this = r;
   return *this;
}

Int64::operator __int64() const
{
   return *reinterpret_cast<const __int64*>(this);
}

char* Int64::toa(char* buf, int base) const
{
	if (base < 2 || base > 36) return 0;
	if (_high >= 0 || base != 10) return UInt64(*this).toa(buf,base);
    char tmp[66];
	UInt64(-(*this)).toa(tmp+1,base);
	tmp[0] = '-';
	if (buf) return strcpy(buf,tmp);
	return strdup(tmp);
}

int Int64::toInt() const
{
   if (!(_high & 0x8000000)) return _low;
   Int64 tmp = -(*this);
   return -(tmp._low);
}

unsigned int Int64::toUInt () const
{
   return _low;
}

long Int64::toLong() const
{
   if (!(_high & 0x8000000)) return _low;
   Int64 tmp = -(*this);
   return -(tmp._low);
}

unsigned long Int64::toULong() const
{
   return _low;
}


Int64 operator+(const Int64& l, const Int64& r)
{
   return Int64(l) += r;
}

Int64 operator-(const Int64& l, const Int64& r)
{
   return Int64(l) -= r;
}

Int64 operator*(const Int64& l, const Int64& r)
{
	int sign = 1;
	if (l._high < 0) sign *= -1;
	if (r._high < 0) sign *= -1;
	UInt64 L = (l._high < 0) ? -l : l;
	UInt64 R = (r._high < 0) ? -r : r;
	UInt64 res = L * R;
	if (sign < 0) return -(*(Int64*) &res);
   return *(Int64*) &res;
}

Int64 operator/(const Int64& l, const Int64& r)
{
   return Int64(l) /= r;
}

Int64 operator%(const Int64& l, const Int64& r)
{
   return Int64(l) %= r;
}

Int64 operator&(const Int64& l, const Int64& r)
{
   return Int64(l) &= r;
}

Int64 operator|(const Int64& l, const Int64& r)
{
   return Int64(l) |= r;
}

Int64 operator^(const Int64& l, const Int64& r)
{
   return Int64(l) ^= r;
}
// ==============================

UInt64::UInt64()
:_high(0),_low(0)
{
}

UInt64::UInt64(unsigned long high, unsigned long low)
:_high(high),_low(low)
{
}

UInt64::UInt64(const UInt64& val)
:_high(val._high),_low(val._low)
{
}

UInt64::UInt64(const Int64& val)
:_high(val._high),_low(val._low)
{
}

UInt64::UInt64(long val)
:_high(val<0 ? (unsigned long)-1 : 0),_low((unsigned long) val)
{
}

UInt64::UInt64(unsigned long val)
:_high(0),_low(val)
{
}

UInt64::UInt64(int val)
:_high(val<0 ? (unsigned long)-1 : 0),_low((unsigned long) val)
{
}

UInt64::UInt64(unsigned int val)
:_high(0),_low(val)
{
}

UInt64::UInt64(short val)
:_high(val<0 ? (unsigned long)-1 : 0),_low((unsigned long) val)
{
}

UInt64::UInt64(unsigned short val)
:_high(0),_low(val)
{
}

UInt64::UInt64(float val)
:_high(0),_low(0)
{
   static const float divisor = (float)(unsigned)0xffffffff + 1.0;
   _low = (unsigned long) fmod(val,divisor);
   _high = (unsigned long) (val / divisor);
}

UInt64::UInt64(double val)
:_high(0),_low(0)
{
   static const double divisor = (double)(unsigned)0xffffffff + 1.0;
   _low = (unsigned long) fmod(val,divisor);
   _high = (unsigned long) (val / divisor);
}

UInt64::~UInt64()
{
}

UInt64& UInt64::operator=(const UInt64& val)
{
   if (&val != this)
   {
      _high = val._high;
      _low = val._low;
   }
   return *this;
}

UInt64& UInt64::operator++()
{
   if (++_low == 0) ++_high;
   return *this;
}

const UInt64 UInt64::pinc()
{
   UInt64 result = *this;
   ++(*this);
   return result;
}

UInt64& UInt64::operator--()
{
   unsigned long l = _low;
   if (--_low > l) --_high;
   return *this;
}
const UInt64 UInt64::pdec()
{
   UInt64 result = *this;
   --(*this);
   return result;
}

UInt64 UInt64::operator~() const
{
   return UInt64(~_high,~_low);
}

UInt64 UInt64::operator-() const
{
   return ++UInt64(~_high,~_low);
}

int UInt64::compare(const UInt64& cmp) const
{
   if (_high < cmp._high) return -1;
   if (_high > cmp._high) return 1;
   if (_low == cmp._low) return 0;
   return (_low > cmp._low) ? 1 : -1;
}

UInt64& UInt64::operator+=(const UInt64& val)
{
   unsigned long l = _low;
   _low += val._low;
   if (_low < l) ++_high;
   _high += val._high;
   return *this;
}

UInt64& UInt64::operator-=(const UInt64& val)
{
   (*this) += -val;
   return *this;
}

UInt64& UInt64::operator&=(const UInt64& val)
{
   _high &= val._high;
   _low &= val._low;
   return *this;
}

UInt64& UInt64::operator|=(const UInt64& val)
{
   _high |= val._high;
   _low |= val._low;
   return *this;
}

UInt64& UInt64::operator^=(const UInt64& val)
{
   _high ^= val._high;
   _low ^= val._low;
   return *this;
}


UInt64& UInt64::operator<<=(unsigned nbits)
{
   nbits &= 0x3F;
   for (unsigned i=0;i<nbits;++i)
   {
      _high <<= 1;
      if (_low & 0x80000000L)
         _high |=1;
      _low <<= 1;
   }
   return *this;
}

UInt64& UInt64::operator>>=(unsigned nbits)
{
   nbits &= 0x3F;
   for (unsigned i=0;i<nbits;++i)
   {
      _low >>= 1;
      if (_high & 1)
         _low |= 0x80000000L;
      _high >>= 1;
   }
   return *this;
}

UInt64 UInt64::operator<<(unsigned nbits) const
{
   return UInt64(*this) <<= nbits;
}

UInt64 UInt64::operator>>(unsigned nbits) const
{
   return UInt64(*this) >>= nbits;
}

UInt64& UInt64::operator*=(const UInt64& val)
{
   if ((_high == 0) && (_low == 0))
     return *this;
   if ((val._high == 0) && (val._low == 0))
   {
      return (*this = val);
   }
   *this = *this * val;
   //UInt64 lop = *this;
   //_high = _low = 0;
   //UInt64 mask = 1;
   //for (int i=0; i< 63;++i)
   //{
   //   if ((val._low & mask._low) || (val._high & mask._high))
   //   {
   //      *this += lop;
   //   }
   //   lop <<= 1;
   //   mask <<= 1;
   //}
   return *this;
}

bool UInt64::_div(const UInt64& dividend, const UInt64& divisor, UInt64& q, UInt64& r)
{
   if (!(divisor._high || divisor._low)) // divisor is zero
   {
         r._high = 0xffffffff;
         r._low = 0xffffffff;
         if (dividend._high || dividend._low) // dividend is not zero
         {
            q = r;
         }
      return false;
   }
   if (!(dividend._high || dividend._low)) // dividend is zero
   {
	   q._high = r._high = 0;
	   q._low = r._low = 0;
       return true;
   }
   UInt64 dend =   dividend;
   UInt64 dor =  divisor;
   q._high = r._high = 0;
   q._low = r._low = 0;

   
   int c = dend.compare(dor);
   if (c == 0)
   {
      q._low = 1;
   }
   else if (c < 0)
   {
      r = dend;
   }
   else
   {
      UInt64 qtmp, dd;
      while (1)
      {
         dd = dor;
         qtmp = 1;
         while ((c = dend.compare(dd <<= 1)) >= 0)
		 {
            qtmp <<= 1;
			if (!(dd._high || dd._low)) break;
		 }
		 if (!(dd._high || dd._low)) dd._high = 0x80000000;
         else dd >>= 1; //qtmp >>=1;
         q += qtmp; dend -= dd;
         c = dend.compare(dor);
         if (c == 0)
         {
            ++q;
            break;
         }
         else if (c < 0)
         {
            r = /*dor -*/ dend;
            break;
         }      
      }
   }
  
   return true;
}

UInt64& UInt64::operator/=(const UInt64& val)
{
   UInt64 q,r;
   _div(*this, val, q, r);
   *this = q;
   return *this;
}

UInt64& UInt64::operator%=(const UInt64& val)
{
   UInt64 q,r;
   _div(*this, val, q, r);
   *this = r;
   return *this;
}

//UInt64& UInt64::operator%=(const Int64& val)
//{
//   UInt64 q,r;
//   _div(*this, val, q, r);
//   *this = r;
//   return *this;
//}

UInt64::operator unsigned __int64() const
{
   return *reinterpret_cast<const __int64*>(this);
}

char* UInt64::toa(char* buf, int base) const
{
	if (base < 2 || base > 36) return 0;
   char btmp[65];
   static const int bufsize = 65;
   if (_high == 0)
   {
	   ultoa(_low,btmp,base);
	   if (buf)
	   {
		   return strcpy(buf,btmp);
	   }
	   return strdup(btmp);
   }
   const UInt64 div = base;
   UInt64 tmp =  *this;
   UInt64 r;
   btmp[bufsize-1] = 0;
   int i;
   for(i=bufsize-2;tmp > 0 && (i>=0);--i)
   {
	   _div(tmp,div,tmp,r);
	   if (r._low < 10)
		   btmp[i] = '0' + (char) r._low;
	   else
		   btmp[i] = 'a' + (char) (r._low - 10);
   }
   if (buf) return strcpy(buf,btmp+i+1);
   return strdup(btmp+i);
}

//const char* UInt64::c_str() const
//{
//   static char buf[30];
//   static const int bufsize = 30;
//   static const UInt64 div = 10;
//   UInt64 tmp = *this;
////   __int64 i64val = *this;
//   bool neg = tmp.toInt() < 0;
//   if (neg) tmp = -tmp;
////   i64val = tmp;
//   buf[bufsize-1] = 0;
//   buf[bufsize-2] = '0';
//   int i;
//   for(i=bufsize-2;(tmp.toInt() > 0) && (i>=0);--i)
//   {
//      char c = '0' + (tmp % div).toInt();
//      buf[i] = c;
//      tmp /= div;
//   }
//   if (neg) buf[i] = '-';
//   else ++i;
//   if (_str) free(_str);
//   _str = strdup(buf+i);
//   return _str;
//}
int UInt64::toInt() const
{
   if (!(_high & 0x8000000)) return _low;
   UInt64 tmp = -(*this);
   return -(tmp._low);
}

unsigned int UInt64::toUInt () const
{
   return _low;
}

long UInt64::toLong() const
{
   if (!(_high & 0x8000000)) return _low;
   UInt64 tmp = -(*this);
   return -(tmp._low);
}

unsigned long UInt64::toULong() const
{
   return _low;
}


UInt64 operator+(const UInt64& l, const UInt64& r)
{
   return UInt64(l) += r;
}

UInt64 operator-(const UInt64& l, const UInt64& r)
{
   return UInt64(l) -= r;
}

UInt64 operator*(const UInt64& l, const UInt64& r)
{
	UInt64 res;
	const unsigned short* L = (const unsigned short*) &l;
	const unsigned short* R = (const unsigned short*) &r;
	for (int i=0;i<4;++i)
	{
		unsigned long li = L[i];
		if (!li) continue;
		for (int j=0;i+j<4 ;++j)
		{
			unsigned long rj = R[j];
			if (!rj) continue;
            res += (UInt64(li*rj)) <<= 16*(i+j);
		}
	}
   return res;
}

UInt64 operator/(const UInt64& l, const UInt64& r)
{
   return UInt64(l) /= r;
}
UInt64 operator%(const UInt64& l, const UInt64& r)
{
   return UInt64(l) %= r;
}

UInt64 operator&(const UInt64& l, const UInt64& r)
{
   return UInt64(l) &= r;
}

UInt64 operator|(const UInt64& l, const UInt64& r)
{
   return UInt64(l) |= r;
}

UInt64 operator^(const UInt64& l, const UInt64& r)
{
   return UInt64(l) ^= r;
}
