#
# Copyright (c) 2001-2002 Daniel van Vugt <danv@users.sf.net>
# This file is part of the Rex library.
# See LICENSE.txt for license details.
#

THISFILE = makefile.gnu
include common.mak
MAKECONFIG2 = $(MAKECONFIG)2
-include $(MAKECONFIG)

PLATFORM = $(shell uname -s)

SHAREDLIBEXT = so
ifeq "$(PLATFORM)" "HP-UX"
SHAREDLIBEXT = sl
endif

STATICNAME = $(LIBNAME)$(STATICSUFFIX)
DYNAMICNAME = $(LIBNAME)$(DYNAMICSUFFIX)
STATICLIB = lib$(STATICNAME).a
DYNAMICLIB0 = lib$(DYNAMICNAME).$(SHAREDLIBEXT)
DYNAMICLIB1 = $(DYNAMICLIB0).$(MAJORVERSION)
DYNAMICLIB2 = $(DYNAMICLIB1).$(MINORVERSION)

ifeq "$(BUILDTYPE)" "debug"
DEBUG = 1
endif

CFLAGS = -DREX_SOURCE

STRIP_DEBUG = echo No debug syms to strip from

ifeq "$(PLATFORM)" "Darwin"
BUILDENV = "GCC"
CC = gcc
DEBUG_CFLAGS = -ggdb
OPTIMIZE_CFLAGS = -O2
CFLAGS += -Wall -pedantic -ansi -fPIC
SHARED_LDFLAGS = -shared
endif

ifeq "$(PLATFORM)" "Linux"
BUILDENV = "GCC"
CC = gcc
DEBUG_CFLAGS = -ggdb
OPTIMIZE_CFLAGS = -O2
CFLAGS += -Wall -pedantic -ansi -fPIC
SHARED_LDFLAGS = -shared -Wl,-h,$(DYNAMICLIB1)
STRIP_DEBUG = strip -g
endif

ifeq "$(PLATFORM)" "SCO_SV"
BUILDENV = "GCC"
CC = gcc
CFLAGS += -Wall -pedantic -ansi -fPIC
DEBUG_CFLAGS = -g
OPTIMIZE_CFLAGS = -O2
SHARED_LDFLAGS = -shared -Wl,-h,$(DYNAMICLIB1)
endif

ifeq "$(PLATFORM)" "SunOS"
BUILDENV = "Sun WorkShop 6"
CC = cc
DEBUG_CFLAGS = -g
OPTIMIZE_CFLAGS = -xO2
CFLAGS += -v -KPIC
SHARED_LDFLAGS = -G -h $(DYNAMICLIB1)
endif

ifeq "$(PLATFORM)" "AIX"
BUILDENV = "VisualAge C++ Professional / C for AIX Compiler, Version 5"
CC = xlc_r
CFLAGS +=
DEBUG_CFLAGS = -g
OPTIMIZE_CFLAGS = -O2
LDFLAGS += -brtl
SHARED_LDFLAGS = -G
endif

ifeq "$(PLATFORM)" "HP-UX"
BUILDENV = "HP aCC"
CC = aCC
CFLAGS += +DAportable +z +p +W749 -Aa
DEBUG_CFLAGS = -g0
OPTIMIZE_CFLAGS = +O2
SHARED_LDFLAGS = -b
endif

ifeq "$(PLATFORM)" "OSF1"
BUILDENV = "Compaq C"
CC = cc
CFLAGS += -pthread -std1 -arch generic -intrinsics
DEBUG_CFLAGS = -g2
OPTIMIZE_CFLAGS = -O2
SHARED_LDFLAGS = -shared
LDFLAGS += -error_unresolved
endif

# Strip the leading zero from the front of MINORVERSION, if any.
NMINORVERSION = $(subst A,,$(subst A0,,A$(MINORVERSION)))

CFLAGS += -DREX_MAJORVERSION=$(MAJORVERSION) -DREX_MINORVERSION=$(NMINORVERSION)
CFLAGS += $(EXTRACFLAGS)
LDFLAGS += -L. $(EXTRALDFLAGS)
ifeq "$(BUILDTYPE)" "release"
CFLAGS += -DNDEBUG $(OPTIMIZE_CFLAGS)
else
CFLAGS += -DDEBUG $(DEBUG_CFLAGS)
endif
AR = ar
ARFLAGS = r

LIBOBJS = $(subst OBJEXT,o,$(subst OBJDIR/,,$(TLIBOBJS)))
TOOLS = demo rat

.PHONY: all clean beforeall package

all: $(MAKECONFIG2) beforeall $(STATICLIB) $(DYNAMICLIB2) $(TOOLS)

# this rule is used to avoid the implicit dependency on $(MAKECONFIG)
$(MAKECONFIG2):
ifndef BUILDTYPE
	@echo $(NOCONFIG)
	@exit 1
else
	@ln $(MAKECONFIG) $(MAKECONFIG2)
endif

beforeall::
	@echo "### Building $(BUILDTYPE)"

debug release: clean
	@echo "### Configuring build"
	@echo "###  type: $@"
	@echo "###    on: $(PLATFORM)"
	@echo "### using: $(BUILDENV)"
	@echo "BUILDTYPE = $@" > $(MAKECONFIG)
	@$(MAKE)

profile:
	@$(MAKE) -f $(THISFILE) EXTRACFLAGS='-pg' EXTRALDFLAGS='-pg' BUILDTYPE=debug

verbose:
	@$(MAKE) -f $(THISFILE) EXTRACFLAGS='-DDEBUGPRINT=1' BUILDTYPE=debug

ifeq "$(OSTYPE)" "sco_unix"
SPECFILE = .sco_gcc_specs
LDFLAGS += -specs=$(SPECFILE)
CFLAGS += -specs=$(SPECFILE)
beforeall:: $(SPECFILE)
clean::
	@-rm -f $(SPECFILE)
# this hack to the gcc specs file makes -shared work to create shared libs
$(SPECFILE):
	@gcc -dumpspecs | sed 's/%{!z\*:-z text}//' > $@
endif

.c.o:
	$(CC) $(CFLAGS) -c -o $@ $<

$(DYNAMICLIB0) $(DYNAMICLIB1) $(DYNAMICLIB2): $(LIBOBJS)
	$(CC) $(LDFLAGS) $(SHARED_LDFLAGS) -o $(DYNAMICLIB1) $(LIBOBJS)
	ln -fs $(DYNAMICLIB1) $(DYNAMICLIB2)
	ln -fs $(DYNAMICLIB1) $(DYNAMICLIB0)
ifndef DEBUG
	$(STRIP_DEBUG) $(DYNAMICLIB2)
endif

$(STATICLIB): $(LIBOBJS)
	$(AR) $(ARFLAGS) $(STATICLIB) $(LIBOBJS)
ifndef DEBUG
	$(STRIP_DEBUG) $(STATICLIB)
endif

TESTTYPE = DYNAMIC
demo: demo.o $($(TESTTYPE)LIB)
	$(CC) $(LDFLAGS) -o $@ demo.o -l$($(TESTTYPE)NAME)
ifndef DEBUG
	$(STRIP_DEBUG) $@
endif

rat: rat.o $($(TESTTYPE)LIB)
	$(CC) $(LDFLAGS) -o $@ rat.o -l$($(TESTTYPE)NAME)
ifndef DEBUG
	$(STRIP_DEBUG) $@
endif

clean::
	@-rm -f *.o \
		*.a \
		*.$(DEBUGEXT) 	\
		$(TOOLS)	\
		$(STATICLIB)	\
		$(DYNAMICLIB0)* \
		$(MAKECONFIG2) \
		so_locations \
		out gmon.out bench
	@echo "" > $(MAKECONFIG)

PACKAGEDIRBASE = /tmp
PACKAGENAME = $(LIBNAME)-$(MAJORVERSION).$(MINORVERSION)
PACKAGEDIR = $(PACKAGEDIRBASE)/$(PACKAGENAME)
DONTPACKAGE = bench.c backup
package: clean
	-rm -rf $(PACKAGEDIRBASE)/$(PACKAGENAME)*
	-mkdir $(PACKAGEDIR)
	cp -R * $(PACKAGEDIR)
	rm -f $(addprefix $(PACKAGEDIR)/,$(DONTPACKAGE))
	cd $(PACKAGEDIRBASE) && \
		tar cv $(PACKAGENAME) | \
			gzip - > $(PACKAGENAME)-src.tar.gz
	cd $(PACKAGEDIRBASE) && \
		zip -r9 $(PACKAGENAME)-src.zip $(PACKAGENAME)
