/*
 * Copyright (c) 2001-2002 Daniel van Vugt <danv@users.sf.net>
 * This file is part of the Rex library.
 * See LICENSE.txt for license details.
 */

#include "rexprivate.h"

parseresult_t
EscapeChar(const REXCHAR *format, int len, REXCHAR *out)
/* assumes that a backslash came before format */
{
	REXCHAR first;

	if (len <= 0)
	{
		return ENCODE_PARSE_ERROR(REX_INCOMPLETE_ESCAPE, 0);
	}

	first = format[0];
	if (first == 'x')
	{
		int i, lim;
		REXCHAR sum;

		lim = (len > 3) ? 3 : len;

		sum = 0;
		i = 1;
		while (i < lim)
		{
			int digit;
			REXCHAR ch;

			ch = format[i];
			if ( (ch >= '0') && (ch <= '9') )
			{
				digit = ch - '0';
			}
			else if ( (ch >= 'A') && (ch <= 'F') )
			{
				digit = ch - 'A' + 0xA;
			}
			else if ( (ch >= 'a') && (ch <= 'f') )
			{
				digit = ch - 'a' + 0xA;
			}
			else
			{
				return ENCODE_PARSE_ERROR(REX_INVALID_ESCAPE, i);
			}
			
			sum = (sum << 4) | digit;
			i++;
		}

		if (lim < 3)  /* We do this after the while to show that  */
		{             /* the error occured after the hex digits   */
			return ENCODE_PARSE_ERROR(REX_INCOMPLETE_ESCAPE, lim);
		}
		*out = sum;
		return 3;
	}
	else
	{
		REXCHAR result;

		switch (first)
		{
			case '0': result = '\0'; break;
			case 'a': result = '\a'; break;
			case 'f': result = '\f'; break;
			case 'n': result = '\n'; break;
			case 'r': result = '\r'; break;
			case 't': result = '\t'; break;
			case 'v': result = '\v'; break;
			default: result = first; break;
		}
		*out = result;
		return 1;
	}
}

